/**
 * Static fields and methods of plugin level
 */
package seat;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchMessages;
import org.eclipse.ui.internal.dialogs.DialogUtil;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.PropertySheetPage;

import seat.model.core.dynamic.Trace;
import seat.ui.tree.viewer.MultiTraceEditor;

/**
 * Static fields and methods at plugin level.
 */
public class SeatUtil {
	// View class names
	public static final String VIEW_REOURCENAVIGATOR="org.eclipse.ui.views.ResourceNavigator";
	public static final String VIEW_PROPERTY="org.eclipse.ui.views.PropertySheet";
	public static final String VIEW_BOOKMARK="org.eclipse.ui.views.BookmarkNavigator";
	public static final String VIEW_MODEL="seat.ui.views.ModelView";
	public static final String VIEW_CONTROLPANEL="seat.ui.views.ControlPanelView";
	public static final String VIEW_PATTERN="seat.ui.views.PatternView";
	public static final String VIEW_UTILITY="seat.ui.views.UtilityView";	
	public static final String SEARCH_PAGE="seat.ui.views.TraceSearchPage";

	// Icons name and their key in image registry
	public static final String ICON_ROOT="class_hi.gif";
	public static final String ICON_PACKAGE="package.gif";
	public static final String ICON_CLASS="class_obj.gif";
	public static final String ICON_COMPOUND="compound.gif";
	public static final String ICON_METHOD="public_co.gif";
	public static final String ICON_FILTER="fields_co.gif";
	public static final String ICON_UTILITY="utility.gif";  
	public static final String ICON_PATTERN="pattern.gif"; 

	// plugin wide color IDs
	public static final String COLOR_HIGHTLIGHT="col.highlight";  	// current hight color realted to view selection
	public static final String COLOR_FILTER="col.filter";		  	// hidden nodes
	public static final String COLOR_UTILITY="col.utility";			// utility color
	public static final String COLOR_PATTERN="col.pattern";			// pattern color
		
	// algorithm exptension point and algorthm  uniques IDs
	public static final String ALGORITHM_EXTENSION_POINT="seat.algorithms";
	public static final int RESERVED_ALGIDX_UNSPECIFY=0; // unused
	public static final int RESERVED_ALGIDX_FILTER=1;    // user selected filter
	public static final int RESERVED_ALGIDX_UTILITY=2;   // utility method, default also hidden 
	public static final int RESERVED_ALGIDX_NO3=3;		 // unused
	public static final int RESERVED_ALGIDX_NO4=4;		 // unused

	// IDs used in prefrence settings
	public static final String TRACE_UTILITY_METHODLIST="pref.utility.methodlist"; //Save utility methods list in store
	public static final String TRACE_SHOWFILTERED="pref.showfiltered";
	public static final String TRACE_SHORTPROPERTY="pref.short";
	public static final String TRACE_ICON="pref.iconmatrix";
	public static final String TRACE_UTILITY_HIDE="pref.util.hide";
	public static final String TRACE_LONG_DECORATION="pref.util.long_decoration";
	public static final String TRACE_DECORATION="pref.util.decoration";
	public static final String TRACE_REPETITION="pref.util.repetition";
	public static final String TRACE_HIDDEN_FLAG="pref.util.hiddenflag";
	public static final String TRACE_ENABLED_DECORATORS="ENABLED_DECORATORS";

	// Trace marker IDs
	public static final String MARKER_EXTENSION_POINT="seat.tracenode";
	public static final String MARKER_EXPLORATION="m.exploration";
	public static final String MARKER_PATH="m.path";

	// diffrnet type of traces, used in property
	public static final int TRACE_OO=3;
	public static final int TRACE_ROUTINE=4;
	public static final int TRACE_C=1;
	public static final int TRACE_JAVA=2;

//	public static final int COLOR_USERSELECT=0;
//	public static final int COLOR_UTILITY=1;
//	public static final int COLOR_PATTERN=2;
//	public static final int COLOR_FILTER=8;




	
//	public static final String TRACE_DIRECTORY="trace.defaultdir";
//	public static final String TRACE_TMP_DIRECTORY="trace.tmpdir";
//	public static final String TRACE_COMPRESS="trace.compress";
//	public static final String TRACE_COMPILER="trace.compiler";
//	
	
//	public static final String TRACE_ICON_ROOT="trace.icon.root";
//	public static final String TRACE_ICON_CONTROL="trace.icon.control";
//	public static final String TRACE_ICON_UITILITY="trace.icon.utility";
//	public static final String TRACE_ICON_PATTERN="trace.icon.pattern";
//	public static final String TRACE_ICON_PACKAGE="trace.icon.package";
//	public static final String TRACE_ICON_CLASS="trace.icon.class";
//	public static final String TRACE_ICON_METHOD="trace.icon.method";
//	public static final String TRACE_ICON_FILE="trace.icon.file";
//	public static final String TRACE_ICON_ROUTINE="trace.icon.routine";
//	public static final String TRACE_ICON_SUBSYSTEM="trace.icon.subsystem";

	
	
//	public static final int METHOD_IMAGE_WIDTH=25;		
//	public static final int METHOD_TEXT_WIDTH=100;		
//	public static final int METHOD_BAR_WIDTH=13;

	public static final int IMG_DEFAULT=0;
	public static final int IMG_ROOT=1;
	public static final int IMG_CLASS=2;
	public static final int IMG_METHOD=3;
	public static final int IMG_UTILITY=4;
	public static final int IMG_PATTERN=5;
	public static final int IMG_FILTERED=6;
	public static final int IMG_PACKAGE=7;
	public static final int IMG_COMPOUND=8;

	
	
	
	/** 
	 * Returns the preference store of SEAT plugin
	 * @return IPreferenceStore
	 */
	public static IPreferenceStore getPluginPreferenceStore() {
		return SeatPlugin.getDefault().getPreferenceStore();
	}
	
	/**
	 * Returns the system default path where trace file will be loaded and saved 
	 * @return String, path
	 */
//	public static String getDefaultTracePath() {
//		 return getPluginPreferenceStore().getString(SeatUtil.TRACE_DIRECTORY);
//	}

	/**
	 * Returns the temporary path where trace file will be saved 
	 * @return String, path
	 */
//	public static String getTemporaryTracePath() {
//		return getPluginPreferenceStore().getString(SeatUtil.TRACE_NONAME);		 
//	}
	public static Color getColorSetting(String id) {
		 RGB rgb=PreferenceConverter.getColor(getPluginPreferenceStore(),id);
		 return new Color(Display.getCurrent(),rgb);
	}

	public static String getStringSetting(String id) {
		 return getPluginPreferenceStore().getString(id);
	}

	public static String getDefaultSetting(String id) {
		 return getPluginPreferenceStore().getDefaultString(id);
	}
	public static boolean getBooleanSetting(String id) {
		 return getPluginPreferenceStore().getBoolean(id);
	}
	public static int getIntegerSetting(String id) {
		 return getPluginPreferenceStore().getInt(id);
	}
	public static void setIntegerSetting(String id, int i) {
		 getPluginPreferenceStore().setValue(id,i);
	}
	
	public static void addToUtilityDepot(String methodFullName) {
		IPreferenceStore store=getPluginPreferenceStore();
		String utilityString=(String)store.getString(SeatUtil.TRACE_UTILITY_METHODLIST);
		int index=utilityString.indexOf(methodFullName);
		if (index==-1)
			utilityString+=methodFullName+ ";";
		Assert.isNotNull(utilityString);
		store.setValue(SeatUtil.TRACE_UTILITY_METHODLIST, utilityString);
	}
	
	public final static String TRACE_NULL="_~null";
	public final static String TRACE_INACTIVE="_~inactive";
	public final static String TRACE_NONAME="_~noname";
	public final static String SPACE_SEPARATOR="#";

	/**
	 * Returns the ImageDescriptor by a file name. The image file is assumed to 
	 * locate in the icons subdirectory. User needs specify: "icons/"+ gif name
	 * @param name - image file name
	 * @return ImageDescriptor used to createe real Image
	 */	
	public static ImageDescriptor getImageDescriptor(String name) {
		String iconPath = "";
		try {
			URL installURL = SeatPlugin.getDefault().getDescriptor().getInstallURL();
			URL url = new URL(installURL, iconPath + name);
			return ImageDescriptor.createFromURL(url);
		} catch (MalformedURLException e) {
			// should not happen
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}

	/**
	 * Returns the workbench shell.
	 * @return top level workbench shell
	 */
	public static Shell getWorkbenchShell() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
	}
	
	/**
	 * Finds the given editor by its editor id. 
	 * @param viewId- A string id of editor to be found.
	 * @return ViewPart that stands for a view.
	 */
	public static IEditorPart getEditor(Trace byTrace) {
		IEditorReference[] refer=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
		Trace contained=null;
		for (int i=0;i<refer.length;i++) {
			IEditorPart edit=refer[i].getEditor(true);
			if (edit instanceof MultiTraceEditor) {
				contained=((MultiTraceEditor)edit).getTrace();
				if (contained==byTrace)
					return edit;
			}
		}
		return null;
	}


//	public static IEditorPart getActiveEditor() {
//		if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()==null)
//			return null;
//		else
//			return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
//	}
	
	public static IEditorPart getActiveEditor() {
		IWorkbenchWindow[] windows=PlatformUI.getWorkbench().getWorkbenchWindows();
		if ((windows!=null)&&windows.length>0) {
			if (windows[0].getActivePage()==null)
				return null;
			else 
				return windows[0].getActivePage().getActiveEditor();
		}
		return null;
	}
	
	public static IWorkbenchPart getActivePart() {
		IWorkbenchWindow[] windows=PlatformUI.getWorkbench().getWorkbenchWindows();
		if ((windows!=null)&&windows.length>0) {
			if (windows[0].getActivePage()==null)
				return null;
			else 
				return windows[0].getActivePage().getActivePart();
		}
		return null;
	}	

	public static void openTraceEditor(IFile file) {
		IWorkbenchPage active=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		if (active==null) return;
		try {
			active.openEditor(new FileEditorInput(file),"seat.ui.tree.partial.PartialTreeEditor");
		} catch (PartInitException e) {
			DialogUtil.openError(
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
				WorkbenchMessages.getString("OpenSystemEditorAction.dialogTitle"), //$NON-NLS-1$
				e.getMessage(),
				e);
		}
	}

	public static IViewPart switchToView(String id) {
		IWorkbenchWindow[] windows=PlatformUI.getWorkbench().getWorkbenchWindows();
		if ((windows!=null)&&windows.length>0) {
			IWorkbenchPage active;
			if (windows[0].getActivePage()==null)
				return null;
			else { 
				active=windows[0].getActivePage();
				if (active==null) return null;
				IViewPart f=active.findView(id);
				if (f!=null) {
					active.bringToTop(f); // not active it, remain current focused window
					return f;
				} 
		 
				try {
					f=active.showView(id);
				} catch (PartInitException pie) {
					return null;
				}
				return f;
			}
		}
		return null;
	}		
	
	/**
	 * Only find reference to this view, not switch it as current
	 * Null will be returned if the view does not exist.
	 * @param id
	 * @return
	 */
	public static IViewPart findView(String id) {
		IWorkbenchPage active=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();		
		if (active==null) return null;
		IViewPart f=active.findView(id);
		return f;  
	}		


	public static void refreshStatisViewIfOpened() {		
//		StatisView statis=(StatisView)SeatUtil.findView("seat.ui.views.StatisView");		
//		if (statis!=null) {
//			statis.refresh();
//		}
	}					

	public static void refreshPropertyViewIfOpened() {		
		PropertySheet prop=(PropertySheet)SeatUtil.findView(IPageLayout.ID_PROP_SHEET);		
		if (prop!=null) {
			IPage page=prop.getCurrentPage();
			if (page instanceof PropertySheetPage)
				((PropertySheetPage)page).refresh();
		}
	}					
	
	
	public static void hideView(String id) {
		IWorkbenchPage active=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();		
		if (active==null) return ;
		IViewPart f=active.findView(id);
		if (f==null) return ; // not there
		active.hideView(f);
	}		
	public static void setStatusMessage(IEditorPart editor,String msg) {
		IStatusLineManager status=editor.getEditorSite().getActionBars().getStatusLineManager();			
		status.setMessage(msg);		
	}
	public static void setStatusMessage(String msg) {
		IEditorPart editor=getActiveEditor();
		IStatusLineManager status=editor.getEditorSite().getActionBars().getStatusLineManager();			
		status.setMessage(msg);		
	}
	
	/**
	 * Finds the first selected object that exists in the primary Viewer UI control. 
	 * This method requires that the ViewPart implements IViewer.
	 * @param viewId- view id
	 * @return first selection, or null if there is no Viewer control.
	 */
	
//	public static Viewer getViewer(String viewId) {
//		ViewPart view=ViewHelper.getView(viewId);		
//		if (view instanceof IViewer) { 
//			Viewer primaryCtrl=((IViewer)view).getViewer();
//			return primaryCtrl;
//		}
//		else 
//			return null;
//	}	
//	
//	public static Object getFirstSelection(String viewId) {
//		ViewPart view=ViewHelper.getView(viewId);		
//		if (view instanceof IViewer) { 
//			Viewer primaryCtrl=((IViewer)view).getViewer();
//			IStructuredSelection selection=(IStructuredSelection)primaryCtrl.getSelection();
//			return selection.getFirstElement();
//		}
//		else 
//			return null;
//	}
//
//	/**
//	 * Returns all the selections in the given view id in an Iterator method.
//	 * This method requires that the ViewPart implements IViewer.
//	 * @param viewId
//	 * @return Iterator for the selection, or null if there is no Viewer control.
//	 */
//	public static Iterator getSelections(String viewId) {
//		ViewPart view=ViewHelper.getView(viewId);		
//		if (view instanceof IViewer) { 
//			Viewer primaryCtrl=((IViewer)view).getViewer();
//			IStructuredSelection selection=(IStructuredSelection)primaryCtrl.getSelection();
//			return selection.iterator();
//		}
//		else 
//			return null;
//	}

	/**
	 * Creates a string from a String array,separated by ';'.
	 * @param items- String array.
	 * @return -Contacted string.  
	 */
	public static String createListString(String[] items) {
		StringBuffer path = new StringBuffer("");
	
		for (int i = 0; i < items.length; i++) {
			path.append(items[i]);
			path.append(";");
		}
		return path.toString();
	}

	/**
	 * conveniece method for displaying a message
	 * @param message-string to be displayed
	 */
	public static void showMessage(String message) {
		MessageDialog.openInformation(null,"Message Window",message);
	}
	/* (non-Javadoc)
	 * @see seat.ui.tree.ITreeNode#inside(java.util.List)
	 */
	public static boolean removeIfInside(Object object,List list) {
		// TODO Auto-generated method stub
		Iterator it=list.iterator();
		while (it.hasNext()) {
			Object next=it.next();
			if ( next.equals(object) ) {
				list.remove(next);
				return true;
			}
		}
		return false;
	}
	
	public static void printTime(String s) {
		long startTime = System.currentTimeMillis();			
		System.out.println(s + startTime/1000);		
	}
	

}
