/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: TraceLoader.java										 *	
 * Package: io										 			 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import seat.model.core.dynamic.Edge;
import seat.model.core.dynamic.MethodInvocationNode;
import seat.model.core.dynamic.MethodInvocationTrace;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Scenario;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.model.core.structure.BehaviouralElement;
import seat.model.core.structure.ClassElement;
import seat.model.core.structure.MethodElement;
import seat.model.core.structure.SoftwareSystem;
import seat.model.core.structure.StreamSource;



public class MethodInvocationTraceLoader extends TraceLoader
{
  private Hashtable labels;
  private Hashtable nodes;
  private Hashtable classes;
  private BufferedReader traceReader;
  private String ln;
  
//  private HashSet parentedNodes;

  /** Creates a method invocation trace loader
   *
   * @param trace the trace to load
   *
   */
    
  public MethodInvocationTraceLoader(Trace trace)
  {
	 super(trace);
  	 
	 labels = new Hashtable();
	 nodes  = new Hashtable();
	 classes= new Hashtable();
	 traceReader = null;
	 ln = "";
	 
//	 parentedNodes=new HashSet();
  }
  
  /** Loads a trace into memory. */

  public void open() 
  {
	try 
	{
	   
	   openTrace(); // open the file
	   skip(); // skips comments and empty lines
	   
	   if (ln.startsWith("#Labels")) 
	   {
		  skip();
  	  	  
		  // loading labels
		  while (!ln.startsWith("#Nodes")) 
		  {
			//System.out.println("add " + ln);
			addLabel();
			skip();
		  }
  	  	  
		  skip();
  	  	  
		  // loading nodes
		  while (!ln.startsWith("#Edges")) 
		  {
			//System.out.println("node " + ln);
			addNode();
			skip();
		  }
  	  	  
		  skip();

		  while (ln != null)
		  {
			//System.out.println("edge " + ln);
			addEdge2();
			skip();
		  }
	   }	
	   
	   traceReader.close();
	   
	   	
	   /* The file was read
		* we need to determine the root of the trace
		*/
		 
	   for (Enumeration e = nodes.keys(); e.hasMoreElements();)
	   {
		  Node node = (Node)nodes.get((String)e.nextElement());
		  if (node.isRoot())
		  {
			((TreeTrace)trace).setRoot(node);
			break;
		  }	
	   }

//	   for (Enumeration e = nodes.keys(); e.hasMoreElements();)
//	   {
//		  Node node = (Node)nodes.get((String)e.nextElement());
//		  if (!parentedNodes.contains(node))
//		  {
//			((TreeTrace)trace).setRoot(node);
//			break;
//		  }	
//	   }	
	// added by Eric Sep 2004 to addd line no and orsered parent edges
	// based on edge lineno
//	traverseGraph(((TreeTrace)trace).getRoot());
//		sortParentEdges();
	final Thread sorterThread = new Thread(new Runnable() {
		public void run() {
			traverseGraph(((TreeTrace)trace).getRoot());
			sortParentEdges();
//			trace.getState().s
			System.out.println("Sorter thread ends.");						

		}
	});
	sorterThread.start();	    
	} 
	catch (IOException e) 
	{
		System.out.println("Problem opening the CTF file");
	}
	catch (IndexOutOfBoundsException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (NullPointerException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (NumberFormatException e)
	{
		System.out.println("Problem in line " + ln);
	}
	catch (Exception e)
	{
		System.out.println("Problem in line " + ln);
	}
  }
 
  /** Adds a label 
   *
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/

  private void addLabel()
	throws IndexOutOfBoundsException, NullPointerException, NumberFormatException
  {
	 ClassElement cl;
	 BehaviouralElement behElem;
	 String label;
	 String labelID;  

	 label   = ln.substring(0, ln.indexOf(",")).trim(); // the label
	 labelID = ln.substring(ln.indexOf(",") + 1).trim(); // label ID
	 cl = null;
	 behElem = null;
	 
	 SoftwareSystem sys = trace.getScenario().getSystem();
	 if (sys.isParsed())
	 {
		cl = (ClassElement)sys.getModelElement(getString(label, 0));
		behElem = (BehaviouralElement)cl.getMethod(getString(label, 1));
	 }
	 else
	{
		// Modified by Eric to cache existent ClassElement
		// Note by Eric for parsing:
		// package string can be parsed from name of cl, which is a full name includes
		// pak1.pak2.ClassName,
		String fullName=getString(label, 0);
		cl= (ClassElement)classes.get(fullName);
		if (cl == null) {
			cl = new ClassElement(sys, fullName);
			classes.put(fullName, cl);
		}				
		behElem = new MethodElement(sys, getString(label, 1), cl);
		
//orginal
//		cl = new ClassElement(sys, getString(label, 0));
//		behElem = new MethodElement(sys, getString(label, 1), cl);
	 }
	 // Attention: The system might not be parsed well. What to do?
	 labels.put(labelID, behElem);
  }
  
  /** Adds a node 
   *
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/
   
  private void addNode()
	throws IndexOutOfBoundsException, NullPointerException, NumberFormatException
  {
	 String cert;    
	 String labelID; 
	 
	 cert = ln.substring(0, ln.indexOf(",")).trim(); // the certificate
	 labelID = ln.substring(ln.indexOf(",") + 1).trim(); // label ID
	 
	 Node node = new MethodInvocationNode((BehaviouralElement)labels.get(labelID), 
										  Integer.parseInt(cert));
	 nodes.put(cert, node);
	 
	 // update the comprehension units table of the trace
	 ((TreeTrace)trace).addComprehensionUnit(node, Integer.parseInt(cert));
  }

  /** Adds an edge 
   * 
   * @throws IndexOutOfBoundsException
   * @throws NumberFormatException
   * @throws NullPointerException
   **/
   
  private void addEdge2()
	throws IndexOutOfBoundsException, NumberFormatException, NullPointerException
  {
	 StringTokenizer allTokens;
	 String cert1;
	 String cert2;
	 
     
	 allTokens = new StringTokenizer(ln, ",");
	 cert1 = (String)allTokens.nextToken().trim(); // the certificate 1
	 cert2 = (String)allTokens.nextToken().trim(); // the certificate 2

	 Node node1 = (Node)nodes.get(cert1);
	 Node node2 = (Node)nodes.get(cert2);
	 
	 Edge edge=new Edge(node1,node2);
	 node1.addChild(node2,edge);
	 /*
	  * Simply use node2.addParent(node1,edge); will not link parent in correct order.
	  * Though in the above statement, the child is linked correctly. Parents that call
	  * a child should be linked in a first call first added fashion.Instead, after main loading is done, a traverse of tree is conducted and parents
	  * are linked then, see: traverseGraph();
	  */
	 node2.addParent(node1,edge);
//	 parentedNodes.add(node2); // to remember which nodes havee parents, to determine ROOT
	

//	 // Look for repetitions due to loop
//	 String repet = null;
//	 if (allTokens.hasMoreTokens())
//	 {
//		repet = (String)allTokens.nextToken().trim();
////		Edge edge = node1.getEdge(node2);
//		edge.setNumberOfRepetitions(Integer.parseInt(repet));
//	 }	
	String repet = (String)allTokens.nextToken().trim();
	edge.setNumberOfRepetitions(Integer.parseInt(repet));
  }
  private void addEdge()
  throws IndexOutOfBoundsException, NumberFormatException, NullPointerException
  {
	int comma= ln.indexOf(',');
	int comma2=ln.indexOf(',',comma+1);
		
	String cert1=ln.substring(0,comma).trim();	
	String cert2=ln.substring(comma+1,comma2).trim();
	String repet=ln.substring(comma2+1).trim();
	Node node1 = (Node)nodes.get(cert1);
	Node node2 = (Node)nodes.get(cert2);
	 
	node1.addChild(node2);
	node2.addParent(node1);
//    edge.setNumberOfRepetitions(Integer.parseInt(repet));
  	
  }
  /** parse the trace file line by line: file_name routine_name nesting_level. */
  private String getString(String ln, int pos)
  {
	StringTokenizer allTokens = new StringTokenizer(ln, " ");
	String[] list = new String[allTokens.countTokens()];
	int i = 0;
	while(allTokens.hasMoreTokens())
	{
		list[i] = allTokens.nextToken();
		i++;
	}		
	return list[pos];
  }
  	
  /** Returns True if the line is empty. */
  private boolean isEmptyLine(String ln)
  {
	 return (ln.length() == 0);
  }
	
  /** Returns True if the line is a comment. */
  
  private boolean isComment(String ln)
  {
	return ln.startsWith("%");
  }
  
  /** Returns True if the line is a command. */
  
  private boolean isCommand(String ln)
  {
	return ln.startsWith("#");
  }
  
  /** Opens the file that corresponds to the trace. */
  
  // Modified on may5,2004 to load from a Source not a String, original name: openFile  
  private void openTrace()
	throws IOException
  {
	// Open the CTF physical file
	ITraceSource source = trace.getTraceSource(); // added on May5,2004
	if (source instanceof StreamSource)
		traceReader = new BufferedReader(new InputStreamReader(((StreamSource)source).getStream()));// added on May5,2004
	else 
		throw new IOException();
  }  
  
  /** Returns True if the file is empty. */
  
  private boolean EOF()
  {
	return (ln == null);
  }
  
  /** Returns the next element of the file. */
  
  private String getNext()
	throws IOException
  {
	 return traceReader.readLine();
  }
  
  /** Skips comments and empty lines. */
  
  private void skip()
	throws IOException
  {
	ln = getNext();
	if (!EOF())
	{
		ln = ln.trim();	
		while(isComment(ln) || isEmptyLine(ln)) 
			ln = getNext();
	 }		
  }

	// Counter for edges removed. edges!=total nodes.
	// Added by Eric to ask for state info from loader
	public Collection getNodes() {
		return nodes.values();
	}
	public int getUniuqeNodes() {
		return nodes.size();
	}
	
	
	/**
	 * This method has two purposes:
	 * 1. set line number for each edge;
	 * 2. link parents for each node
	 */
	// set line no info
	private int lineNo=2; // inv-0,main-1
//	public void traverseEdges(Node root) {
//		traverseEdges(root,startLineNo);
//	}
	public void traverseGraph(Node root) {
		for (int i=0;i<root.getNumberOfChildren();i++) {
			Edge edge=root.getEdge(i);
			
			// Set line number
			if (edge.getStartingLineNumber()==0)
				edge.setStartingLineNumber(lineNo);
			int rep=edge.getNumberOfRepetitions();
			if (rep==0)
				rep=1;
			lineNo+=rep;
			
			// set parents of second node in edge
//			Node second=edge.getSecondNode();
//			second.addParent(root,edge);
			
			// depth first for children nodes
			traverseGraph(root.getChildAt(i));			
		}
	}
	
	/**
	 * Sorts parent nodes accroding to the edge line no. Therefore,
	 * This method must be called after traverseGraph() which sets line nos.
	 */
	public void sortParentEdges() {
		for (Enumeration e = nodes.keys(); e.hasMoreElements();) {
			Node node = (Node) nodes.get((String) e.nextElement());
			Vector pe=node.getParentEdgeVector();
			Edge[] edges=(Edge[])pe.toArray(new Edge[pe.size()]);
			Arrays.sort(edges, new Comparator() {
				public int compare(Object o1,Object o2) {
					Edge e1=(Edge)o1;
					Edge e2=(Edge)o2;
					return e1.getStartingLineNumber()-e2.getStartingLineNumber();
				}
			});
			pe.clear();
			for (int i=0;i<edges.length;i++)
				pe.add(edges[i]);
		}
	}
	
	public static void main(String[] args) {
		long startTime=0,endTime=0;
		startTime = System.currentTimeMillis();
			
		SoftwareSystem system=new SoftwareSystem("IDE","test ide for dev");
		Scenario sce=new Scenario(system,"Update Database"); 
		MethodInvocationTrace trace=new MethodInvocationTrace(sce,new StreamSource("c:\\trace3.ctf"));
		MethodInvocationTraceLoader loader = new MethodInvocationTraceLoader(trace);
		loader.open();

		endTime = System.currentTimeMillis();
		System.out.println("Trace(graph) load time: " + (endTime - startTime)/1000.0 +"sec");
		loader.save("c:\\t3.ctf");
	}

	
	/** Saves the trace in the form of CTF
	  *
	  * @param file full path of the file that will contain the trace
	  */ 
	  public void save(String file) 
	  {
		try {
			BufferedWriter writer= new BufferedWriter(new FileWriter(file));
			Node root=((TreeTrace)trace).getRoot();
			gatherNodes(root);
		
			writer.write("#Labels (Label, Label_ID)\n");
			Iterator it=nodeSet.iterator();
			Node next=null;
			Hashtable labelTable=new Hashtable();
			int labelCounter=1; 
			while (it.hasNext()) {
				next=(Node)it.next();
				String label=next.getLabel().toString();
				if (!labelTable.containsKey(label)) {			
					labelTable.put(label,new Integer(labelCounter++));
					int lastDot=label.lastIndexOf(".");
					String first,second;
					if (lastDot!=-1) {
						first=label.substring(0,lastDot);
						second=label.substring(lastDot+1);
					} else {
						first="N";
						second=label;
					}
					writer.write("\n"+first+" "+second+","+labelTable.get(label));
				}
			}

			writer.write("\n\n#Nodes (Node_ID, Label_ID)\n");
			it=nodeSet.iterator();
			while (it.hasNext()) {
				next=(Node)it.next();
				writer.write("\n"+next.getCertificate()+","+labelTable.get(next.getLabel().toString()));
			}
		  		
			writer.write("\n\n#Edges (Node_ID, Node_ID, [Edge_Label])\n");
			it=nodeSet.iterator();
			while (it.hasNext()) {
				next=(Node)it.next();
				Node child=null;
				for (int i=0;i<next.getNumberOfChildren();i++) {
					child=next.getChildAt(i);
					writer.write("\n"+next.getCertificate()+","+child.getCertificate()+","+next.getEdge(i).getNumberOfRepetitions());		  	
				}
			}
			writer.close();
		  		
		} catch (IOException ioe) {
			System.out.println("an IO exception occurred when saving trace!");
		}
	
	  }
  
	  //-----------------Save---------------
	  private HashSet nodeSet=new HashSet();
	  private void gatherNodes(Node root) {
		  if (!nodeSet.contains(root)) {
			 nodeSet.add(root);
			 for (int i=0;i<root.getNumberOfChildren();i++) 
				gatherNodes(root.getChildAt(i));
		  }
	  }
  

}
