/*
 * Created on Sep 8, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.io;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* SEAT : Software Exploration and Analysis Tool
 * 
 * Authors: Abdelwahab Hamou-Lhadj and Lianjiang Fu
 *
 * Supervisor: Dr. Timothy Lethbridge
 *
 * Copyright: KBRE Group - University of Ottawa
 *
 * Version: 0.1
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

public class TraceConverter
{
  private BufferedReader traceReader; 
  private Hashtable table;
  private String lastLineRead;
  private CallNode root;
  private int ln=0;
  /** Creates a tree-based trace loader.
	*
	* @param trace the trace to load
	*/  
  class CallNode
  {
	  String id;
	  int nl;
	  Vector children;
	  Vector parents;
	  int cert;
	
	  CallNode(String fileName, String rtnName, int nl)
	  {
		  this.id = fileName + " " + rtnName;
		  this.nl = nl;
		  this.children = null;
		  this.parents = null;
		  this.cert = 0;
	  }
	
	  String computeSig()
	  {
		  StringBuffer buffer = new StringBuffer(id);
		  if (children==null) 
			  return buffer.toString();
		  for (int i = 0; i < children.size(); i++)
		  {
			  CallNode child = (CallNode)children.elementAt(i);
			  buffer.append("#" + child.cert);
		  }
		  return buffer.toString();
	  }
  }
  
      
  public TraceConverter(String inputFile)
  {
	this.table = new Hashtable();
	try 
	{
		traceReader = new BufferedReader(new FileReader(inputFile));
	} 
	catch(FileNotFoundException e) 
	{
		e.printStackTrace();
	}
  }

  // Reads a line if there are any.
  private boolean hasNext() throws IOException
  {
	lastLineRead = traceReader.readLine();
	return (lastLineRead != null);
  }

  // Returns the node object based on last line read
   private CallNode getNext() //for toad
  {try {
  	ln++;
	int firstSpace = lastLineRead.indexOf(" ");
	int secondSpace = lastLineRead.indexOf(" ", firstSpace + 1);
	int thirdSpace = lastLineRead.indexOf(" ", secondSpace + 1);
	int fourSpace = lastLineRead.indexOf(" ", thirdSpace + 1);
	int fifthSpace = lastLineRead.indexOf(" ", fourSpace + 1);
	String fileName=lastLineRead.substring(secondSpace+1,thirdSpace);
	String rtnName=lastLineRead.substring(thirdSpace+1,fourSpace);
	int nl=Integer.parseInt(lastLineRead.substring(fifthSpace+1));
	return new CallNode(fileName,rtnName,nl);
  } catch (Exception e ) {
  	System.out.println("Line No "+ln+" : "+lastLineRead);
  	return null;
  }
}

//   private CallNode getNext() 
//  {
//	int firstSpace = lastLineRead.indexOf(" ");
//	int secondSpace = lastLineRead.indexOf(" ", firstSpace + 1);
//	int thirdSpace = lastLineRead.indexOf(" ", secondSpace + 1);
//	String fileName=lastLineRead.substring(firstSpace+1,secondSpace);
//	String rtnName=lastLineRead.substring(secondSpace+1,thirdSpace);
//	int nl=Integer.parseInt(lastLineRead.substring(thirdSpace+1));
//	return new CallNode(fileName,rtnName,nl);
//}

  // remove children links that refers to given node
  private void unlinkChildren(CallNode unwanted) 
  {
	if (unwanted.children==null)
		return;
	Iterator it=unwanted.children.iterator();
	while (it.hasNext()) {
		CallNode child=(CallNode)it.next();
		if (child.parents!=null)
			child.parents.remove(unwanted);
	}
  }

	public void save(String output) {
		try {
			BufferedWriter traceWriter= new BufferedWriter(new FileWriter(output));
			while(hasNext()) {
				if (lastLineRead.startsWith("<inst>")&&(!lastLineRead.trim().equals("")))
					traceWriter.write(lastLineRead+"\n");				
			}
			traceWriter.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}				
	}

  /**
	*Open the trace, create the internal graph
	*/
  public void open()
  {
//	CallNode root = null;
	Stack stack;
	int cert = 0;
	int size = 0;
 
	try 
	{
	   CallNode current = null;
	   CallNode prev = null;
	 
	   if (hasNext())  // Test if the trace is not empty
	   {
		 root = getNext(); // first node should be the root
		 size++;
		 prev = root;      // save the previous node
	 	
		 stack = new Stack(); // create a stack
		 stack.push(root); 	 // push the first node into the stack

		 // look for other nodes
		 while (hasNext()) 	
		 {
			current = getNext(); // Get the current node
			if (current==null) // in case of exception
			 	continue;
			size++;

			int diff = current.nl - prev.nl;			
			if (diff <= 0) 
			{
			   while (((CallNode)stack.peek()).nl >= current.nl) 
			   {
				  CallNode node = (CallNode)stack.pop();
				  String sig = node.computeSig();
		  	   	
				  if (!table.containsKey(sig))
				  {
					table.put(sig, node);
					node.cert = cert;
					cert++;	
				  }
				  else
				  {
					node = (CallNode)table.get(sig);
				  }
			   		
				  CallNode parent = (CallNode)stack.pop();
				  if (parent.children==null)
					  parent.children=new Vector(10);
				  parent.children.addElement(node);
				  if (node.parents==null) 
					  node.parents=new Vector(10);
				  node.parents.addElement(parent);
				  stack.push(parent); 
				}
			} 
			stack.push(current);	
			prev = current;		
		  }

		  while (!stack.empty()) 
		  {
			 CallNode node = (CallNode)stack.pop();
			 String sig = node.computeSig();
		  	   	
			 if (!table.containsKey(sig))
			 {
				table.put(sig, node);
				node.cert = cert;
				cert++;	
			 }
			 else
			 {
				 node = (CallNode)table.get(sig);
			 }
			   		
			 if (!stack.empty())
			 {
				CallNode parent = (CallNode)stack.pop();
				if (parent.children==null)
					parent.children=new Vector(10);
				parent.children.addElement(node);
				if (node.parents==null) 
					node.parents=new Vector(10);
				node.parents.addElement(parent);
				stack.push(parent); 
			 }	
		  }
	   }
	   System.out.println("Size : " + size);
	   System.out.println("Table size : " + table.size());
	 }
	 catch (Exception e)
	 {
		e.printStackTrace();
	 }  
  }

	public void saveAs2(String output) {
	try {
		BufferedWriter traceWriter= new BufferedWriter(new FileWriter(output));
		
		Hashtable labelTable=new Hashtable(50);
		Hashtable nodeTable=new Hashtable(50);
		int labelIndex=1;
		String key,label;
		CallNode node,child;
		int certificate;
		Enumeration en=table.keys();
		while (en.hasMoreElements()) {
			key=(String)en.nextElement();
			int sep=key.indexOf('#');
			label=(sep==-1)?key:key.substring(0,sep);
			
			if (!labelTable.containsKey(label)) {
				labelTable.put(label,new Integer(labelIndex));
				labelIndex++;
			}
			nodeTable.put((CallNode)table.get(key),labelTable.get(label));
		}
	
		en=labelTable.keys();		
		while (en.hasMoreElements()) {
			key=(String)en.nextElement();
			traceWriter.write( key + "," + labelTable.get(key)+"\n");
		}
		traceWriter.close();
		

		traceWriter= new BufferedWriter(new FileWriter(output+".node"));
		en=nodeTable.keys();		
		while (en.hasMoreElements()) {
			node=(CallNode)en.nextElement();
			traceWriter.write(node.cert  + "," + nodeTable.get(node)+"\n");
		}		
		traceWriter.close();

		// *** There is an extra \n at the start of edge,
		// remove it in loading
		traceWriter= new BufferedWriter(new FileWriter(output+".edge"));
		en=nodeTable.keys();		
		while (en.hasMoreElements()) {
			node=(CallNode)en.nextElement();
			if (node!=root)
				saveNode(node,traceWriter);
		}		
		saveNode(root,traceWriter);
		traceWriter.close();
		
	} catch(IOException ioe) {
		ioe.printStackTrace();
	}		
  } 

  public void saveAs(String output) {
  try {
	  BufferedWriter traceWriter= new BufferedWriter(new FileWriter(output));
		
	  Hashtable labelTable=new Hashtable(50);
	  Hashtable nodeTable=new Hashtable(50);
	  int labelIndex=1;
	  String key,label;
	  CallNode node,child;
	  int certificate;
	  Enumeration en=table.keys();
	  while (en.hasMoreElements()) {
		  key=(String)en.nextElement();
		  int sep=key.indexOf('#');
		  label=(sep==-1)?key:key.substring(0,sep);
			
		  if (!labelTable.containsKey(label)) {
			  labelTable.put(label,new Integer(labelIndex));
			  labelIndex++;
		  }
		  nodeTable.put((CallNode)table.get(key),labelTable.get(label));
	  }
	
	  traceWriter.write("\n\n#Labels (Label, Label_ID)\n\n");
	  en=labelTable.keys();		
	  while (en.hasMoreElements()) {
		  key=(String)en.nextElement();
		  traceWriter.write(key + "," + labelTable.get(key)+"\n");
	  }
		
	  traceWriter.write("\n\n#Nodes (Node_ID, Label_ID)\n\n");
	  en=nodeTable.keys();		
	  while (en.hasMoreElements()) {
		  node=(CallNode)en.nextElement();
		  traceWriter.write(node.cert  + "," + nodeTable.get(node)+"\n");
	  }		
		
	  traceWriter.write("\n\n#Edges (Node_ID, Node_ID, [Edge_Label])\n\n");
	  en=nodeTable.keys();		
	  while (en.hasMoreElements()) {
		  node=(CallNode)en.nextElement();
		  if (node!=root)
			  saveNode(node,traceWriter);
	  }		
	  saveNode(root,traceWriter);
		
	  traceWriter.close();
  } catch(IOException ioe) {
	  ioe.printStackTrace();
  }		
} 


	class NodeCount {
		CallNode node;
		int count;
		NodeCount(CallNode node,int count) {
			this.node=node;
			this.count=count;
		}
	}
	private void saveNode(CallNode node,BufferedWriter traceWriter) {
		Vector childVect=new Vector(50);
		CallNode current=null,prev=null;
		if (node.children!=null) { // has children
			for (int i=0;i<node.children.size();i++) {
				current=(CallNode)node.children.get(i);
				if (current!=prev) 
					childVect.add(new NodeCount(current,1));
				else {
					NodeCount last=(NodeCount)childVect.lastElement();
					last.count++;
				}
				prev=current;					
			}
			
			try {
				NodeCount nc;
				for (int j=0;j<childVect.size();j++) {
					nc=(NodeCount)childVect.get(j);
					traceWriter.write( node.cert  + "," + nc.node.cert + "," + nc.count + "\n");
				}
			} catch(IOException ioe) {
					ioe.printStackTrace();
			}											
		}
	}
	
  public static void main(String[] args) 
  {
  	String s="d:\\TT567";
  	TraceConverter tc=new TraceConverter(s+".trace");
//  	tc.save(s+".temp");
  	tc.open();
  	tc.saveAs(s+".ctf");
	System.out.println("Done: "+s);
  }
}  



