/*
 * Created on May 6, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.io;

import java.util.ArrayList;
import java.util.Hashtable;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;

/**
 * Internal invariable state information of the trace. These states
 * are gathered and set during trace loading and can be queried. 
 */
public class TraceState {
	private Trace trace;
	
	private float loadTime;  		// loading time in seconds
	private int type;				// type of trace
	private ITraceSource source;    // trace source file	
	private int graphSize=0;		// size(nodes) of internla graph model
	private int treeNodes=0;		// total tree nodes
	private int compressedNodes=0;	// current hidden nodes
	private int detectedPatterns=0; // no of paterns in trace

	private ArrayList packageList=new ArrayList();  // constructed thru parsing, not sorted 
	private ArrayList classList=new ArrayList();    // not sorted, Model Ele: LogicalPackage/ClassElement
	private ArrayList nodeFinalList=new ArrayList(); //sorted by name

	private Hashtable initialNodeStates;
	// for source parsing
	private Hashtable javaClasses=new Hashtable();  // for JavaElements from java core parsing
	
	private Hashtable subtreeSize=new Hashtable();  // record size of all subtree for fast performance
	/**
	 * 
	 */
	public TraceState(Trace trace) {
		this.trace=trace;
		loadTime=0.0f;
		type=SeatUtil.TRACE_C;
		source=null;
	}

	/**
	 * @return
	 */
	public int getDetectedPatterns() {
		return detectedPatterns;
	}

	/**
	 * @return
	 */
	public int getGraphSize() {
		return graphSize;
	}

	/**
	 * @return
	 */
	public float getLoadTime() {
		return loadTime;
	}

	/**
	 * @return
	 */
	public ITraceSource getSource() {
		return source;
	}

	/**
	 * @return
	 */
	public int getTreeNodes() {
		return countSubtree(((TreeTrace)trace).getRoot());
	}



	/**
	 * @return
	 */
	public int getCompressedNodes() {
		return countSubtreeHidden(((TreeTrace)trace).getRoot());
	}
	

	/**
	 * @return
	 */
	public int getType() {
		return type;
	}

	/**
	 * @param i
	 */
	// set in DetectPattern.apply/undo method
	public void setDetectedPatterns(int i) {
		detectedPatterns = i;
	}

	/**
	 * @param i
	 */
	// set in trace.open(); 
	public void setGraphSize(int i) {
		graphSize = i;
	}

	/**
	 * @param f
	 */
	// set in trace.open(); 
	public void setLoadTime(float f) {
		loadTime = f;
	}

	/**
	 * @param source
	 */
	// set in trace constructor
	public void setSource(ITraceSource source) {
		this.source = source;
	}

	/**
	 * @param i
	 */
	public void setType(int i) {
		type = i;
	}


	private int countSubtree(Node node) {
		Object size=subtreeSize.get(node);
		if (size!=null) { // already calculated
			return ((Integer)size).intValue();
		}
		int counter=1;
		for (int i=0;i<node.getNumberOfChildren();i++) {
			int rep=node.getEdge(i).getNumberOfRepetitions();
			if (node.getChildAt(i).isLeaf())
				counter+=rep;
			else
				counter+=rep*countSubtree(node.getChildAt(i));
		}
		subtreeSize.put(node,new Integer(counter));
		return counter;		
	}

	private int countSubtreeHidden(Node node) {
		if (node.isHidden())
			return countSubtree(node);
		else {
			int counter=0;
			for (int i=0;i<node.getNumberOfChildren();i++) {
				int rep=node.getEdge(i).getNumberOfRepetitions();
				if (node.getChildAt(i).isHidden())
					counter+=rep*countSubtree(node.getChildAt(i));
				else 
					counter+=rep*countSubtreeHidden(node.getChildAt(i));
			}
			return counter;			
		}
	}
	
	
	/**
	 * @return
	 */
	public final Hashtable getJavaClasses() {
		return javaClasses;
	}

	/**
	 * @return
	 */
	public final ArrayList getPackageList() {
		return packageList;
	}

	/**
	 * @return
	 */
	public final ArrayList getClassList() {
		return classList;
	}
	/**
	 * This is array should NOT ne modified.
	 * @return
	 */
	public final ArrayList getModelNodeList() {
		return nodeFinalList;
	}	
	
	/**
	 * Gets initial trace model states after it first loaded.
	 * This method needs to be called just after loading, otherwise
	 * data contained will not be relevant.
	 * Assumption: This method will only compute node first time it runs.
	 * It does NOT consider model nodes adding and deleting. Because in 
	 * that case, if the state of a node is initial or not can not be determined.
	 * Therefore, In that scenario, the user must modify this returned hashtable.   
	 */
	public Hashtable getInitialNodeStates() {
		if (initialNodeStates==null) {
			initialNodeStates=new Hashtable();
			for (int i=0;i<nodeFinalList.size();i++) {
				Node node=(Node)nodeFinalList.get(i);
				initialNodeStates.put(node,new Integer(node.getBits()));
			}
		}
		return initialNodeStates;
			
	}
//	/**
//	 * @return
//	 */
//	public IProject[] getProjects() {
//		return srcProjects;
//	}
//	/**
//	 * @param projects
//	 */
//	public void setProjects(IProject[] projects) {
//		srcProjects = projects;
//	}
//
//	/**
//	 * @return
//	 */
//	public IProject getActiveProject() {
//		return activeProject;
//	}
//	/**
//	 * @param project
//	 */
//	public void setActiveProject(IProject project) {
//		activeProject = project;
//	}
}