/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: DetectPAtterns.java								     *	
 * Package: algorithms										     *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.algorithms;

import java.util.Enumeration;

import seat.io.TraceState;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Pattern;
import seat.model.core.dynamic.PatternOccurrence;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.algorithm.Parameter;

//May 5, 2004 Eric, for all algorithms that need to process single node, implements ICompressNode
// change parameter form TreeTrace to Trace type 

// note for improvement: After applying ignore order of children algorithm, one node can becomes 'same'
// as some other nodes. But internally, this node is only changed with it certificate. Parent/children
// relations are intact. In this case, it is not added as a pattern even it should be because of the transform.
// future imple needs to consider these category of new nodes: igonre order, ignore repetition,( state =1,2):
public class DetectPatterns extends Algorithm
{
  private static TreeTrace trace;  // Trace
  private static int threshold;    // Determines number of repetition of a sequence

 
  /* (non-Javadoc)
   * @see seat.ui.algorithm.IAlgorithm#init(seat.ui.algorithm.Parameter[])
   */
  public void init(Parameter[] parameters) {
	super.init(parameters);
	// TODO Auto-generated method stub
	for (int i = 0; i < parameters.length; i++)
		if (parameters[i].getID().equalsIgnoreCase("seat.threshold")) {
			try {
				threshold = Integer.parseInt(parameters[i].getCurrent());
			} catch (NumberFormatException nfe) {
				threshold = 2;
				return;
			}
			break;
		}
  }

 
  /** Detects patterns that occur in a non-contiguous way in a tree based trace
   *
   * @param args not required for this algorithm
   */
	
  public void apply(Trace inputTrace)
  {
  	trace = (TreeTrace)inputTrace;
  	
  	//mark, threshold is set in init(), this line is here only fore reference
  	//threshold = Criteria.patternThreshold;
  	
  	//**************Add by Eric to clear orginal Nov 27 2003**** 
  	// note: which is a pattern:
  	//1. not leaf(has children);2. has more parents
	undo(trace);  	
    
    findPatterns(trace.getRoot());

	// May 10, 2004 , Eric ,set state information in trace    
	TraceState state=trace.getState();
	state.setDetectedPatterns(trace.getNumberOfPatternOccurrences());    
  }

  public void undo(Trace inputTrace) {
  	Enumeration en=((TreeTrace)inputTrace).getPatternOccurrences();
  	PatternOccurrence po;
  	Pattern pattern;
  	Node initiator; 
  	while (en.hasMoreElements()) {
  		po=(PatternOccurrence)en.nextElement();
  		pattern=po.getPattern();  // removie this from Pattern list
  		pattern.removePatternOccurrence(po);
  		initiator=po.getInitiator();// move it from node
  		initiator.setPatternOccurrence(null);  		 
  	}
	((TreeTrace)inputTrace).removeAllPatternOccurrence();
	
	TraceState state=trace.getState();
	state.setDetectedPatterns(trace.getNumberOfPatternOccurrences());	
  }

  public static void process(Trace inputTrace)
   {
	 trace = (TreeTrace)inputTrace;

// only for future reference, do NOt uncomment.
//	 threshold = Criteria.patternThreshold;
  	
	 //**************Add by Eric to clear orginal Nov 27 2003**** 
	 // note: which is a pattern:
	 //1. not leaf(has children);2. has more parents
	 unprocess((TreeTrace)trace);  	
    
	 findPatterns(trace.getRoot());
   } 
   
   public static void unprocess(Trace inputTrace) {
	 Enumeration en=((TreeTrace)inputTrace).getPatternOccurrences();
	 PatternOccurrence po;
	 Pattern pattern;
	 Node initiator; 
	 while (en.hasMoreElements()) {
		 po=(PatternOccurrence)en.nextElement();
		 pattern=po.getPattern();  // removie this from Pattern list
		 pattern.removePatternOccurrence(po);
		 initiator=po.getInitiator();// move it from node
		 initiator.setPatternOccurrence(null);  		 
	 }
	 ((TreeTrace)inputTrace).removeAllPatternOccurrence();
   }
  private static void findPatterns(Node node)
  {
  	//**************Add by Eric to clear orginal Nov 27 2003****
  	// to clear the reulst of last run 
//  	node.setPatternOccurrence(null);
  	
  	
  	if (node.isLeaf()||node.isHidden())
  		return;
  	// get to see if all children are hidden. if yes, this node equals to a leaf
  	Enumeration en=node.getChildren();
  	int shown=0;
  	Node child;
  	while (en.hasMoreElements()) {
  		child=(Node)en.nextElement();
  		if (!child.isHidden()) {
  			shown++;
  			break;
  		}
  	}
  	if (shown==0)  // all children are hidden
  		return;
  		
  	if (node.getNumberOfParents() >= threshold && !node.isInitiator())
  	{
  		// New pattern is detected
  		PatternOccurrence pttOcc = new PatternOccurrence(trace, 
  														 new Pattern(), 
  														 node, 
  														 node.getNumberOfParents());
  		return;
    }	
    
    for (Enumeration e = node.getChildren(); e.hasMoreElements();)
    	findPatterns((Node)e.nextElement());
  }	


}  