/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: IgnoreOrder.java							 	 		 *	
 * Package: algorithms										     *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.algorithms;

import java.util.Enumeration;
import java.util.Hashtable;

import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;
import seat.ui.algorithm.ICompressNode;

public class IgnoreOrder extends Algorithm implements ICompressNode {
	public static Hashtable table;
	// Global table for computing new certificates
	private static Hashtable visitedCompUnits; // Visited comprehenison units

	public void apply(Trace trace) {
		table = new Hashtable();   // this two new cannot be put into init() because undo doesn't want them cleared
		visitedCompUnits = new Hashtable();
		Criteria.ignoreOrder=true; // add by Eric
		Node node = ((TreeTrace) trace).getRoot();
		ignoreOrderHelper(node);
	}

	public void undo(Trace trace) {
		Criteria.ignoreOrder=false; // add by Eric
		Node node = ((TreeTrace) trace).getRoot();
		restoreOrderHelper(node);
	}

	/**
	 * @param node node that will undergo the changes 
	 */

	public void process(Node node) {
		table = new Hashtable();
		visitedCompUnits = new Hashtable();
		Criteria.ignoreOrder=true; // add by Eric
		ignoreOrderHelper(node);
	}

	/** Helps perfoming the ignore-order procedure. */
	private void ignoreOrderHelper(Node node) {
		if (node.isLeaf())
			return;

		for (Enumeration e = node.getChildren(); e.hasMoreElements();) {
			Node child = (Node) e.nextElement();
			if (!visitedCompUnits.containsKey(child)) {
				ignoreOrderHelper(child);
				visitedCompUnits.put(child, "");
			}
		}

		String sig = node.computeSignature();

		if (table.containsKey(sig)) {
			Node nn=(Node)table.get(sig);  // modi by Eric
			int cert = nn.getCertificate();//Integer.parseInt((String) table.get(sig));
			node.setCertificate(cert);
			node.setState(1);  // can NOT set node to hide
		} else {
			table.put(sig, node);//modi by Eric //String.valueOf(node.getCertificate()));
		}
	}

	public void unprocess(Node node) {
		Criteria.ignoreOrder=false; // add by Eric
		restoreOrderHelper(node);
	}

	/** Helps perfoming the ignore-order procedure. */

	private void restoreOrderHelper(Node node) {
		if (node.isUnordered()) {
			node.setCertificate(TreeTrace.getCertificate(node));
			node.setState(0);
		}

		for (Enumeration e = node.getChildren(); e.hasMoreElements();) {
			Node child = (Node) e.nextElement();
			restoreOrderHelper(child);
		}
	}

}