/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: Edge.java									 		     *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

 
package seat.model.core.dynamic;


// May6 2204 add: implemetns IEdge,
// also change signature
public class Edge
{
  private Node firstNode;	
  private Node secondNode;
  private int numberOfRepetitions;
  private int startingLineNumber=0;  // Add by Eric

  /** Creates an edge.  */
    
  public Edge(Node firstNode, Node secondNode) 
  {
  	this.firstNode = firstNode;
  	this.secondNode = secondNode;
  	this.numberOfRepetitions = 1;  // modi by Eric, instead of 0
  }

  /** Creates an edge with the number of repetitions.  */
    
  public Edge(Node firstNode, Node secondNode, int numberOfRepetitions) 
  {
  	this.firstNode = firstNode;
  	this.secondNode = secondNode;
  	this.numberOfRepetitions = numberOfRepetitions;
  }

  /** Returns the first node.  */
    
  public Node getFirstNode() 
  {
  	return firstNode;
  }

  /** Returns the second node. */
    
  public Node getSecondNode() 
  {
  	return secondNode;
  }

  /** Sets the number of repetitions due to loops by count. 
   *
   * @param count number of contiguous repetitions of the node
   *
   */
    
  public void setNumberOfRepetitions(int count)
  {
  	this.numberOfRepetitions = count;
  }
  
  /** Returns the number of repetitions. */
    
  public int getNumberOfRepetitions() 
  {
  	return numberOfRepetitions;
  }
  
  /** Removes this edge. */
  public void remove()
  {
  	firstNode = null;
  	secondNode = null;
  }
  
  // added by Eric for node line no in source trace file
/**
 * @return
 */
public int getStartingLineNumber() {
	return startingLineNumber;
}

/**
 * @param i
 */
public void setStartingLineNumber(int i) {
	startingLineNumber = i;
}


}
