/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: Trace.java											 *	
 * Package: core.dynamic										 *
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.dynamic;

import java.io.IOException;
import java.util.Observable;

import seat.io.ITraceSource;
import seat.io.TraceState;

// May6,2004, Eric changed Source file to
// ITraceSource source and also in subsequent subclass-TreeTrace,MethodInTrace,RoutineTrace
// ITracesource is also included inthe state class  
public abstract class Trace extends Observable
{
  private Scenario scenario;    // The scenario that is traced	
  private ITraceSource source;   // Trace source file
	protected TraceState state; // internal state info, including source 	
  /** Creates a trace 
   *
   * @param scenario the system scenario that corresponds to this trace
   * @param file physical source file 
   *
   */
    
  public Trace(Scenario scenario, ITraceSource source) 
  {
  	this.scenario = scenario;
  	this.source= source;
  	this.state=new TraceState(this);
  	this.state.setSource(source);
  }
  
  /** Returns the scenario that correponds to this trace
   *
   * @return scenario that corresponds to the trace
   *
   */
    
  public Scenario getScenario()
  {
  	return scenario;
  }
 
  /** Returns the source file
   *
   * @return an object of SourceFile
   */

// changed on may6 to return ITraceSource instead of SourceFile 
  public ITraceSource getTraceSource()
  {
 	return source;
  }
  
  /** Loads the trace into memory. 
   * this method needs to be overriden by subclasse. 
   *
   **/
   
  public abstract void open() throws IOException;
  
  //**************added for observables, called by inherit child
  public void setChanged() {
	super.setChanged();
  }
  public void clearChanged() {
	super.clearChanged();
  }
  
	/**
	 * @return
	 */
	public TraceState getState() {
		return state;
	}

}