/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: LogicalPackage.java									 *
 * Package: core.structure										 *	
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.core.structure;

import java.util.Vector;
import java.util.Enumeration;

public class LogicalPackage extends StructuralElement
{
  private Vector structuralElements; // Structural elments that belong to this package
 
  /** Creates a logical package object by specifying the parent package and package name 
   * 
   * @param pck logical package that contains this package.
   * @param name name of the package
   *
   */ 
  
  public LogicalPackage(SoftwareSystem sys, LogicalPackage pck, String name)
  { 	
 	super(sys, pck, name);
 	pck.addStructuralElement(this);
 	structuralElements = new Vector();
  }
 
  /** Creates a logical package object by specifying the package name only
   * 
   * @param name name of the package
   *
   */ 
  
  public LogicalPackage(SoftwareSystem sys, String name)
  { 	
 	super(sys, null, name);
 	structuralElements = new Vector();
  }
 
  /** Adds a structural element to this package.
   * 
   * @param strucElem an object of StructuralElement
   *
   */
   
  public void addStructuralElement(StructuralElement structElem)
  {
  	structuralElements.addElement(structElem);
  }

  /** Removes a structural element from this package.
   * 
   * @param strucElem an object of StructuralElement
   *
   */
   
  public void removeStructuralElement(StructuralElement structElem)
  {
  	structuralElements.removeElement(structElem);
  	structElem.setPackage(null);
  } 
  
  /** Returns the number of structural elements that exist in this package
   * 
   * @return number of structural elements of this package
   *
   */
  
  public int getNumberOfStructuralElements()
  {
  	return structuralElements.size();
  }
    
  /** Returns all structural elements of this package.
   * 
   * @param an enumeration that contains all structural elements of this package
   *
   */
  
  public Enumeration getStructuralElements()
  {
  	return structuralElements.elements();
  }
}