/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: MethodElement.java									 *	
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.structure;

import java.util.Enumeration;

public class MethodElement extends BehaviouralElement
{
  private ClassElement defClass;   // Represents the defining class of this method
  private boolean constructorFlag; // Flag that determines if this is a constructor or not
  private boolean destructorFlag;  // Flag that determines if this is a destructor or not
  
  // add on June 2004, by Eric, for comment or descfription of a method
  private String Description;
  
  
  // Public attributes that determine the modifiers of this method
  public boolean PUBLIC    = false;  	 	 
  public boolean PRIVATE   = false;  	 	 
  public boolean PROTECTED = false;
  public boolean STATIC    = false;
  public boolean ABSTRACT  = false;
  public boolean FINAL     = false;

  /** Creates a method object
   * 
   * @param name of the method
   * @param defClass class that defines this method
   *
   */
   
  public MethodElement(SoftwareSystem sys, String name, ClassElement defClass)
  { 	
 	super(sys, name);
 	this.defClass = defClass;
 	this.defClass.addMethod(this);
 	this.constructorFlag = false;
 	this.destructorFlag = false;
 	
 	// June 2004 Eric
 	this.Description="";
  }
  
  /** Returns the class that defines this method
   * 
   * @return an object of ClassElement
   *
   */
   
  public ClassElement getClassElement()
  {
  	return defClass;
  }
  
  /** Sets the class that defines this method. It is used within the package only
   * 
   * @param defClass class that defines this method
   *
   */

  void setClassElement(ClassElement defClass)
  {
  	this.defClass = defClass;
  }
  
  /** Sets the constructor flag to true if this method is a constructor
   * 
   * @param true if this method is a constructor, false otherwise
   *
   */

  public void setConstructorFlag(boolean flag)
  {
	this.constructorFlag = flag;
  }

  /** Checks whether this method is a constructor or not
   * 
   * @return true if this method is a constructor
   *
   */

  public boolean isConstructor()
  {
	return constructorFlag; 	
  }
 
  /** Sets the destructor flag to true if this method is a destructor
   * 
   * @param true if this method is a destructor, false otherwise
   *
   */

  public void setDestructorFlag(boolean flag)
  {
	this.destructorFlag = flag;
  }

  /** Checks whether this method is a destructor or not
   * 
   * @return true if this method is a destructor, false otherwise
   *
   */

  public boolean isDestructor()
  {
	return destructorFlag; 	
  }
 
  /** Return true is this method overloads another method within the same class
   * 
   * @return true if this method overloads another method in the same class, false otherwise
   *
   */
   
  public boolean overloads()
  {
  	 int count = 0;
  	   	 
  	 for(Enumeration e = defClass.getMethods(); e.hasMoreElements();)
  	 {
  	 	MethodElement methElem = (MethodElement)e.nextElement();
  	 	if (methElem.getName().equalsIgnoreCase(getName()))
  	 	{
  	 		count++;
  	 		if (count > 1)
  	 			return true;
  	 	}	
  	 }
  	 
  	 return false;	
  }
  
  /** Goes through the superclasses of the class that defines this method and 
   * return true is this method overrides another method. Java classes are not included 
   * 
   * @return true if this method overrides anothe method, false otherwise
   *
   */
  
  public boolean overrides()
  {
  	 int count = 0;
  	 
  	 for(Enumeration e = defClass.getSuperclasses(); e.hasMoreElements();)
  	 {
  	     ClassElement classElem = (ClassElement)e.nextElement();
  	     
  	 	 for(Enumeration e1 = classElem.getMethods(); e1.hasMoreElements();)
  	     {
  	 	    MethodElement methElem = (MethodElement)e1.nextElement();
  	 	    if (methElem.getName().equalsIgnoreCase(getName()))
  	 	    {
  	 		   count++;
  	 		   if (count > 0)
  	 		   		return true;
  	 		}  
  	     }
  	 }
  	 return false;
  }
  
  /** Returns the signature of this method. This signature is defined by concatening 
   * the signature of the class that defines it with the method name
   * 
   * @return signature of this method
   *
   */
   
  public String getSignature()
  {
  	return defClass.getName() + "." + getName();
  }


//Eric
/**
 * @return
 */
public String getDescription() {
	return Description;
}

/**
 * @param string
 */
public void setDescription(String string) {
	Description = string;
}

}