/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: ModelElement.java										 *	
 *																 *
 * Author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * Version: 1.0 - July 2003										 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/
 
package seat.model.core.structure;

public abstract class ModelElement
{
  private String name; 		    // Name of the model element
  private StreamSource file;	    // File that defines the model element
  private SoftwareSystem sys;	// Software system that contains this model element
 
  /** Creates a model element object by specifying its name 
   * 
   * @param sys software system that contains this model element
   * @param name name of the model element
   */
   
  public ModelElement(SoftwareSystem sys, String name)
  { 	
 	this.name = name;
 	this.sys = sys;
 	sys.addModelElement(this);
  }

  /** Returns the model element name
   * 
   * @return the model element name
   *
   */
   
  public String getName()
  {
    return name;
  }

  /** Sets the source file that defines this model element
   * 
   * @param file source file 
   *
   */
   
  public void setSourceFile(StreamSource file)
  {
    this.file = file;
  }

  /** Returns the source file that defines this model element
   * 
   * @return an object of the class SourceFile 
   *
   */
   
  public StreamSource getSourceFile()
  {
    return file;
  }


}
