/*****************************************************************
 * Tool: SEAT - Software Exploration and Analysis Tool			 *
 *																 *
 * Class: StructuralElement.java								 *	
 *																 *
 * @author: Abdelwahab Hamou-Lhadj								 *
 *																 *	
 * Affiliation: Knowledge Based Reverse Engineering Group		 *
 *				SITE - University of Ottawa						 *
 *																 *
 * @version: 1.0 - July 2003									 *
 * 																 *
 * Modifications: 												 *
 * (please add date of modifications and author after this line) *	
 *																 *
 *****************************************************************/

package seat.model.core.structure;

public abstract class StructuralElement extends ModelElement
{
  private LogicalPackage pck;  // Package that defines this structural element
 
  /** Creates a structural element object by specifying the package and the name
   * 
   * @param pck logical package that contains this structural element.
   * @param name name of the structural element.
   *
   */
 
  public StructuralElement(SoftwareSystem sys, LogicalPackage pck, String name)
  { 	
 	super(sys, name);
 	this.pck = pck;
  }
 
  /** Creates a structural element object by specifying the name only
   * 
   * @param name name of the structural element.
   *
   */

  public StructuralElement(SoftwareSystem sys, String name)
  { 	
 	super(sys, name);
 	this.pck = null;
  }
 
  /** Returns the package that contains this structural element
   * 
   * @return an object of the class LogicalPackage
   *
   */

  public LogicalPackage getPackage()
  {
 	return pck;
  }
  
  /** Sets the package that contains this structural element
   * 
   * @param pck the package that defines this structural element.
   *
   */
// add 'public' by Eric  
  public void setPackage(LogicalPackage pck)
  {
 	this.pck = pck;
  }

}
