/*
 * Created on Oct 31, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.actions;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.texteditor.MarkerUtilities;

import seat.SeatUtil;
import seat.ui.tree.partial.UniquePath;
import seat.ui.tree.viewer.TraceEditorPage;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class BookMarkAction extends TraceEditorAction implements IViewUpdater{
	private static final String fMarkerType=SeatUtil.MARKER_EXTENSION_POINT;
	private static final String EDIOT_ID_ATTR="org.eclipse.ui.editorID";
	
	public BookMarkAction(TraceEditorPage edit) {
		super(edit,Action.AS_PUSH_BUTTON); 
		setText("Add Bookmark...");
		setToolTipText("Add BookMark");
		setImageDescriptor(SeatUtil.getImageDescriptor("icons/bookmark.gif"));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		IStructuredSelection selection=(IStructuredSelection)page.getViewer().getSelection();
		if (selection==null)
			return;
		Iterator it=selection.iterator();
		while (it.hasNext()) 
			addMarker((UniquePath)it.next());
			
		// switch to utility view
		updateView(SeatUtil.VIEW_BOOKMARK,false);
	}
	
	/**
	 * Creates a new marker according to the specification of this action and
	 * adds it to the marker resource.
	 */
	protected void addMarker(UniquePath node) {
		IResource resource= getResource();
		if (resource == null)
			return;
		Map attributes= getInitialAttributes(node);		
//		if (!askForLabel(attributes))
//			return;
		try {
			MarkerUtilities.createMarker(resource, attributes, fMarkerType);
		} catch (CoreException x) {
			System.out.println("Can not add given marker!"); //$NON-NLS-1$
		}
	}
	
	/**
	 * Removes the given markers.
	 *
	 * @param markers the markers to be deleted
	 */
	protected void removeMarkers(final List markers) {
		try {
			getResource().getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					for (int i= 0; i < markers.size(); ++i) {
						IMarker marker= (IMarker) markers.get(i);
						marker.delete();
					}
				}
			}, null);
		} catch (CoreException x) {
			System.out.println("Can not remove given markers!"); //$NON-NLS-1$
		}
	}	
	/**
	 * Returns the attributes with which a newly created marker will be
	 * initialized.
	 *
	 * @return the initial marker attributes
	 */
	protected Map getInitialAttributes(UniquePath node) {
		
		Map attributes= new HashMap(11);
		
		int line= 999;
		attributes.put(EDIOT_ID_ATTR, "seat.ui.tree.viewer.MultiTraceEditor"); // used by bookmark framework
		attributes.put(IMarker.MESSAGE, node.getFullQualifiedName());
		attributes.put(IMarker.LINE_NUMBER, new Integer(node.getLineNo())); 
		attributes.put(SeatUtil.MARKER_EXPLORATION, ""); 
		attributes.put(SeatUtil.MARKER_PATH, ((UniquePath)node).serialize()); 
//		attributes.put(TraceSearchOperation.LABEL_ATTRIB, node.getFullQualifiedName()); 
//		attributes.put(TraceSearchOperation.TYPE_ATTRIB,new Integer(node.getElement().isLeaf()?1:2));  // 1 for a leaf, 2:subtree
//		attributes.put(IMarker.LINE_NUMBER, line+""); 
		return attributes;
	}	
	
	/** 
	 * Returns the resource for which to create the marker, 
	 * or <code>null</code> if there is no applicable resource.
	 *
	 * @return the resource for which to create the marker or <code>null</code>
	 */
	protected IResource getResource() {
		IEditorInput input= page.getEditorInput();
		
		IResource resource= (IResource) input.getAdapter(IFile.class);
		
		if (resource == null)
			resource= (IResource) input.getAdapter(IResource.class);
			
		return resource;
	}
	/**
	 * Asks the user for a marker label. Returns <code>true</code> if a label
	 * is entered, <code>false</code> if the user cancels the input dialog.
	 * Sets the value of the attribute <code>message</code> in the given
	 * map of attributes.
	 *
	 * @param attributes the map of attributes
	 * @return <code>true</code> if the map of attributes has successfully been initialized 
	 */
	protected boolean askForLabel(Map attributes) {

		Object o= attributes.get("message"); //$NON-NLS-1$
		String proposal= (o instanceof String) ? (String) o : ""; //$NON-NLS-1$
		if (proposal == null)
			proposal= ""; //$NON-NLS-1$

		String title= "Add Bookmark";
		String message="Description for this node";
		IInputValidator inputValidator = new IInputValidator() {
			public String isValid(String newText) {
				return (newText == null || newText.trim().length() == 0) ? " " : null; //$NON-NLS-1$
			}
		};
		InputDialog dialog= new InputDialog(page.getSite().getShell(), title, message, proposal, inputValidator);
		
		String label= null;
		if (dialog.open() != Window.CANCEL)
			label= dialog.getValue();

		if (label == null)
			return false;

		label= label.trim();
		if (label.length() == 0)
			return false;

		MarkerUtilities.setMessage(attributes, label);
		return true;
	}	

	/* (non-Javadoc)
	 * @see seat.ui.views.IViewUpdater#updateView(java.lang.String, boolean)
	 */
	public void updateView(String viewId, boolean makeViewCurrent) {
		// TODO Auto-generated method stub
		IViewPart view;
		if (makeViewCurrent)
			view=SeatUtil.switchToView(viewId);
		else 
			view=SeatUtil.findView(viewId);
	}	
}
