/*
 * Created on Sep 21, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.actions;

import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;

import seat.SeatUtil;
import seat.model.algorithms.Algorithm;
import seat.model.core.dynamic.Node;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.ICompressNode;
import seat.ui.algorithm.Parameter;
import seat.ui.tree.partial.TreeNode;
import seat.ui.tree.viewer.TraceEditorPage;
import seat.ui.views.ModelView;

/**
 * This class is used for Actions tht contributed to the context menu of
 * a trace editor page. A CompressAction will process a single subtree 
 * which is passed from current selection.
 */
public class CompressAction extends TraceEditorAction implements IViewUpdater{
	private AlgorithmDescription algorithmDesc;
		
	public CompressAction(TraceEditorPage page,AlgorithmDescription alg) {
		super(page,Action.AS_CHECK_BOX);
		algorithmDesc=alg;
		setText(alg.getName());
		setImageDescriptor(SeatUtil.getImageDescriptor(alg.getIcon()));
	}		
		
	public int getAlgorithmIndex() {
		String index=algorithmDesc.getIndex();
		return Integer.parseInt(index);
	}
		
	public void run() {
		IStructuredSelection selection=(IStructuredSelection)page.getViewer().getSelection();
		if (selection==null)
			return;
		TreeNode[] paths=new TreeNode[selection.size()];
		paths=(TreeNode[])selection.toList().toArray(paths);
		
		Algorithm algorithm=algorithmDesc.getAlgorithm();		
		if (algorithm instanceof ICompressNode) {
			// prepare and init parameters
			Parameter[] paras=algorithmDesc.getParameters();
			String index=algorithmDesc.getIndex();
			for (int i=1;i<paras.length;i++) { // o is for internal index
				paras[i].setCurrent(SeatUtil.getStringSetting(index+"@#@"+paras[i].getID()));
			}
			algorithm.init(paras);	// prepare and set parameters for compress.process/unprocess	
		
			// compress in a loop
			ICompressNode compress=algorithmDesc.getCompressSubtree();
			int algIndex=Integer.parseInt(algorithmDesc.getIndex());
			for (int i=0;i<paths.length;i++) {  			
				if (this.isChecked())  // just checked, need to hide
					compress.process(paths[i].getElement());
				else  
					compress.unprocess(paths[i].getElement());
				page.getExploration().cacheNodeContext(paths[i],algIndex,this.isChecked());				
			}
			page.getMultiPageEditor().refresh();
			updateView(SeatUtil.VIEW_MODEL,true);	
		}					
	}				
	/* (non-Javadoc)
	 * @see seat.ui.views.IViewUpdater#updateView(java.lang.String, boolean)
	 */
	public void updateView(String viewId, boolean makeViewCurrent) {
		// TODO Auto-generated method stub
		IViewPart view;
		if (makeViewCurrent)
			view=SeatUtil.switchToView(viewId);
		else 
			view=SeatUtil.findView(viewId);
		if (view==null)
			return;

		if (view instanceof ModelView) {
			ArrayList hiddenAr=new ArrayList();
			ArrayList shownAr=new ArrayList();
			ArrayList allNodes=page.getTrace().getState().getModelNodeList();					
			for (int i=0;i<allNodes.size();i++) {
				Node n=(Node)allNodes.get(i);
				if (n.isHidden())
					hiddenAr.add(n);		
				else 
					shownAr.add(n);		
			}
			ModelView model=(ModelView)view;
			model.partActivated((IWorkbenchPart)page.getMultiPageEditor());
			model.hideAndShowNode(hiddenAr.toArray(),shownAr.toArray());	
		}			
		
	}

}