package seat.ui.actions;

import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.ui.tree.partial.TreeNode;
import seat.ui.tree.viewer.TraceEditorPage;
import seat.ui.views.ModelView;
import seat.ui.views.UtilityView;

/**
 * Marks selections nodes in the editor to be utility nodes.
 */
public class MarkUtilityAction extends TraceEditorAction implements IViewUpdater{
	public MarkUtilityAction(TraceEditorPage edit) {
		super(edit,Action.AS_CHECK_BOX);
		setText("Mark as Utility Method");
		setToolTipText("Mark as Utility Method");
		setImageDescriptor(SeatPlugin.getImageDescriptor(SeatUtil.ICON_UTILITY));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		IStructuredSelection selection=(IStructuredSelection)page.getViewer().getSelection();
		if (selection==null)
			return;
		TreeNode[] paths=new TreeNode[selection.size()];
		paths=(TreeNode[])selection.toList().toArray(paths);
		ArrayList util=page.getExploration().getUtilityList();
		for (int i=0;i<paths.length;i++) {
			if ( paths[i]!=null) {
				Node n=paths[i].getElement();
				if (this.isChecked()) {
					if  (!paths[i].inside(util)) {
						util.add(n);	
						n.hide(SeatUtil.RESERVED_ALGIDX_UTILITY);			
					}
				} else {
					util.remove(n);	
					n.show(SeatUtil.RESERVED_ALGIDX_UTILITY);			
				}
				page.getExploration().cacheNodeContext(paths[i],SeatUtil.RESERVED_ALGIDX_UTILITY,this.isChecked());
			}
		}
		
		page.getMultiPageEditor().refresh();
		updateView(SeatUtil.VIEW_UTILITY,true);
		updateView(SeatUtil.VIEW_MODEL,false);
	}
	/* (non-Javadoc)
	 * @see seat.ui.views.IViewUpdater#updateView(java.lang.String, boolean)
	 */
	public void updateView(String viewId, boolean makeViewCurrent) {
		// TODO Auto-generated method stub
		IViewPart view;
		if (makeViewCurrent)
			view=SeatUtil.switchToView(viewId);
		else 
			view=SeatUtil.findView(viewId);
		if (view==null)
			return;

		if (view instanceof UtilityView) {
			((UtilityView)view).updateView(page.getExploration().getUtilityList());			
		} else if (view instanceof ModelView) {
			ArrayList hiddenAr=new ArrayList();
			ArrayList shownAr=new ArrayList();
			ArrayList allNodes=page.getTrace().getState().getModelNodeList();					
			for (int i=0;i<allNodes.size();i++) {
				Node n=(Node)allNodes.get(i);
				if (n.isHidden())
					hiddenAr.add(n);		
				else 
					shownAr.add(n);		
			}
			ModelView model=(ModelView)view;
			model.partActivated((IWorkbenchPart)page.getMultiPageEditor());
			model.hideAndShowNode(hiddenAr.toArray(),shownAr.toArray());	
		}			

	}

}		

