
package seat.ui.actions;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.part.MultiPageEditorActionBarContributor;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.AlgorithmList;
import seat.ui.tree.viewer.MultiTraceEditor;
import seat.ui.tree.viewer.TraceEditorPage;
import seat.ui.views.MethodView;
import seat.ui.views.ModelView;

/**
 * This class is the main tool bar for the plugin. It will generate one action
 * for each algorithm in plugin.xml. The status(on/off) of each algorithm will 
 * be recorded in the list of active exploration which resides inside the active
 * editor page. 
 * 
 * The setActiveEditor() method will be called when there is a change of MultiTraceEditor,
 * and the setActivePage() metrhod will be called when there is a changge of page within
 * a MultiTraceEditor. Therefore, synchronization between trace and exploration is done
 * inside these two method. 
 */
public class MultiActionContributor extends MultiPageEditorActionBarContributor {
	private static AlgorithmAction[] actions;
	private static AlgorithmDescription[] ads;
	private static GroupHideAction groupHideAction;
	
	private TraceEditorPage previousPage;
	/**
	 * Creates all the algorithm actions. 
	 */
	public MultiActionContributor() {
		super();
		// TODO Auto-generated constructor stub	
		createActions();
	}
	
	/*
	 *  Creates algorithm actions from descriptions
	 */	
	 private static void createActions() {
		ads=SeatPlugin.getAlgorithmDescriptions();
		actions=new AlgorithmAction[ads.length];
		for (int i=0;i<ads.length;i++) { 
			actions[i]=new AlgorithmAction(ads[i]);			
		}	
		groupHideAction=new GroupHideAction();			 
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void contributeToMenu(IMenuManager menuManager) {
		// TODO Auto-generated method stub
		super.contributeToMenu(menuManager);
		IMenuManager mainBar = new MenuManager("Algorithms");
		menuManager.insertAfter("additions", mainBar); // addt additions group
		for (int i=0;i<actions.length;i++) 
			if (ads[i].getTargetView().equalsIgnoreCase(SeatUtil.VIEW_MODEL))
				mainBar.add(actions[i]);

		mainBar.add(new Separator());
		for (int i=0;i<actions.length;i++) 
			if (!ads[i].getTargetView().equalsIgnoreCase(SeatUtil.VIEW_MODEL))
				mainBar.add(actions[i]);						

		mainBar.add(new Separator());
		mainBar.add(groupHideAction);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(org.eclipse.jface.action.IToolBarManager)
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager) {
		// TODO Auto-generated method stub
		super.contributeToToolBar(toolBarManager);

		for (int i=0;i<actions.length;i++) 
			if (ads[i].getTargetView().equalsIgnoreCase(SeatUtil.VIEW_MODEL))
				toolBarManager.add(actions[i]);

		toolBarManager.add(new Separator());
		for (int i=0;i<actions.length;i++) 
			if (!ads[i].getTargetView().equalsIgnoreCase(SeatUtil.VIEW_MODEL))
				toolBarManager.add(actions[i]);						

		toolBarManager.add(new Separator());
		toolBarManager.add(groupHideAction);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorActionBarContributor#init(org.eclipse.ui.IActionBars, org.eclipse.ui.IWorkbenchPage)
	 */
	public void init(IActionBars bars, IWorkbenchPage page) {
		// TODO Auto-generated method stub
		super.init(bars, page);
		// There is no active editor at this step yet. Move code to setActiveEditor()
	}
	
	/**
	 * A new editor is activated. Synchronizes trace and exploration. 
	 * 
	 * (non-Javadoc)
	 * Method declared on IEditorActionBarContributor, set check/uncheck status of tool bar
	 */
	public void setActiveEditor(IEditorPart editor) {
		// Run super.
		super.setActiveEditor(editor);
		// Note: an activeEditor will always call setActivepage() 
		// so the following code is marked.
//		setActivePage(editor);
	}
	
	/**
	 * A new editor page is activated. Synchronizes trace and exploration. 
	 * (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorActionBarContributor#setActivePage(org.eclipse.ui.IEditorPart)
	 */
	public void setActivePage(IEditorPart editor) {
		// TODO Auto-generated method stub
		TraceEditorPage newPage=null;
		if (editor instanceof MultiTraceEditor) {
			newPage=((MultiTraceEditor)editor).getCurrentEditor();
			sync(previousPage,newPage);
			previousPage=newPage;		
		} else if (editor instanceof TraceEditorPage) {
			newPage=(TraceEditorPage)editor;
			sync(previousPage,newPage);
			previousPage=newPage;		
		}
	}
		
	private void sync(TraceEditorPage oldPage,TraceEditorPage newPage) {
		if (previousPage!=newPage) {
			ModelView mv=(ModelView)SeatUtil.findView(SeatUtil.VIEW_MODEL);
			MethodView pv=(MethodView)SeatUtil.findView(SeatUtil.VIEW_PATTERN);
			MethodView uv=(MethodView)SeatUtil.findView(SeatUtil.VIEW_UTILITY);
			MultiTraceEditor.firePageChange(oldPage,newPage,mv,pv,uv);
			syncToolbar(newPage);
		}
	}
	/*
	 * Refreshes toolbar status(on/off) with data from the exploration in the given editor page  
	 */	
	private void syncToolbar(TraceEditorPage page) { 
		AlgorithmList al=page.getExploration().getAlgorithmList();
		for (int i=0;i<actions.length;i++) 
			actions[i].setChecked(al.getAlgorithmStatus(ads[i].getIndex()));
	}
	/**
	 * Re-run all algoirthms that are already run to reflect changes, such as a parameter change
	 * in the system. This method calls reapply4CurrentEditor(AlgorithmAction except) with a null
	 * parameter. 
	 */
	public static void reapply4CurrentEditor() {
		reapply4CurrentEditor(null);
	}
	
	/**
	 * Re-runs all the algorithm excepts the given one to reflects changes.
	 * This method exists because some algorithms are inter-related. 
	 * Changes from one algorithm may need refresh of other algorithms.
	 * For example, DetectPattern algorithm will have different results
	 * if some nodes are hidden.  
	 * @param except: algorithm to be excluded. This is to eliminate 
	 * 					recursive effects.
	 */
	public static void reapply4CurrentEditor(AlgorithmAction except) {
		final AlgorithmAction excluded=except;
		BusyIndicator.showWhile(Display.getCurrent(),new Runnable() {
			public void run() {
				IEditorPart ep=SeatUtil.getActiveEditor();
				Assert.isNotNull(ep);
				if (ep instanceof MultiTraceEditor) {
					MultiTraceEditor med=(MultiTraceEditor)ep;
					if (actions==null) 
						createActions();
					for (int kk=0;kk<actions.length;kk++) { // iterate all the actions except the give current one
						if (actions[kk]!=excluded) 				
							actions[kk].core(med.getCurrentEditor(),true);
					} // for kk
					med.refresh();	
						
				}
			}
		});
			
	}
	


}
