package seat.ui.algorithm;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.dynamic.TreeTrace;

/**
 * A filtering algorithm that hidden nodes that are global utility.
 * Global utility is system level and applies to all traces that the SeatPlugin can handle.
 * Any nodes that matches any global utilities will need initially be set to hidden status.    
 * 
 * This classes will be called after trace is first loaded. (In TraceEditor-loadTrace())
 */
public class GlobalUtilityFilter implements IAlgorithm, ICompressNode {
	private static String ul;
	
	/**
	 * 
	 */
	public GlobalUtilityFilter() {
		super();
		// TODO Auto-generated constructor stub
		init(null);
	}

	public void apply(Trace trace) {
		Node node = ((TreeTrace) trace).getRoot();
		remove(node, true);
	}
	public void undo(Trace trace) {
		// dup.clear();
		Node node = ((TreeTrace) trace).getRoot();
		remove(node, false);
	}
	public void process(Node node) {
		remove(node, true);
	}
	public void unprocess(Node node) {
		remove(node, false);
	}

	private void remove(Node node, boolean apply) {
		for (int i = 0; i < node.getNumberOfChildren(); i++) {
			Node child = (Node) node.getChildAt(i);
			if (ul.indexOf(child.getLabel())!=-1) {
				if (apply)
					child.hide(SeatUtil.RESERVED_ALGIDX_UTILITY);
				else
					child.show(SeatUtil.RESERVED_ALGIDX_UTILITY);
			} else {
				remove(child, apply);
			}
		}
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.IAlgorithm#init(seat.ui.algorithm.Parameter[])
	 */
	public void init(Parameter[] parameters) {
		// TODO Auto-generated method stub
		ul=SeatUtil.getStringSetting(SeatUtil.TRACE_UTILITY_METHODLIST);
	}

}