/*
 * Created on May 4, 2004
 *
 *  */
package seat.ui.algorithm;

import seat.model.core.dynamic.Trace;

/**
 * Interface that all algorithms must follow to be loaded by the framwork.
 * Abstract class Algorithm implements this interface and realized init to get 
 * unique index. 
 */
public interface IAlgorithm {
	/**
	 * Initializes an algorithm with given parameters. 
	 * This method must be extended(not override) if an algorithm
	 * needs parameters. super.init(...) needs always be called so 
	 * an algorithm can always get its index.  
	 * @param parameters
	 */
	public void init(Parameter[] parameters);
	
	/**
	 * Runs the algorithm on the given trace
	 * @param trace
	 */
	public void apply(Trace trace);  // run algorithm to hide nodes
	
	/**
	 * Undos the effects of the algorthm.
	 * @param trace
	 */
	public void undo(Trace trace);   // revoke 
}
