package seat.ui.algorithm;

import java.io.File;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.eclipse.core.resources.IProject;

import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.structure.ClassElement;
import seat.model.core.structure.LogicalPackage;
import seat.model.core.structure.MethodElement;

/**
 * This class parses .class files exists in the given trace projects and constructs
 * all necessary relations, including hierarchy and method attributes.
 * Only those class that participate in the trace run-time will be parsed.
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class JavaClassParser extends Parser {
	private Hashtable classElements= new Hashtable();

	private ArrayList urlList;
	private String fileExtension; // Extension of the files to parse (.class or .java)
	private String logFile; 		// Log File

	private int numberOfParsedClasses; // Number of classes the system contains
	private Vector errNotParsedClasses; // Contains the list of classes that are not parsed
	private int numberOfMethods = 0;// number of method
	
	public JavaClassParser(Trace trace, IProject active, IProject[] sourcePrj) {
		super(trace, active, sourcePrj);
		// TODO Auto-generated constructor stub
//		javaClasses = trace.getState().getJavaClasses();
//		this.system = system;
		this.numberOfParsedClasses = 0;
		this.fileExtension = ".class";
		this.logFile = "report.txt";
		this.errNotParsedClasses = new Vector();
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#hookMoreParsing()
	 */
	public void hookMoreParsing() {
		// TODO Auto-generated method stub
		super.hookMoreParsing();
		// Write report
		System.out.println(
			"Number of classes parsed in the system  +    : " + numberOfParsedClasses);
		System.out.println(
			"Number of methods parsed in the system  +    : " + numberOfMethods);
		System.out.println(
			"Number of classes not successfully parsed: " + errNotParsedClasses.size());

		if (errNotParsedClasses.size() != 0) {
			System.out.println(
				"The list of classes that are not parsed (not found):");
			for (int i = 0; i < errNotParsedClasses.size(); i++)
				System.out.println(
					(String) errNotParsedClasses.get(i));
		}		
	}		
	
	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setPackageClassRelation()
	 */
	public void setPackageClassRelation() {
		// TODO Auto-generated method stub
		Hashtable pck=new Hashtable();
		Hashtable cls=new Hashtable();

		ArrayList nl=trace.getState().getModelNodeList();
		String prevPckClass="";
		String pckClassName,lpn,lcn;
		int lastDot=-1;
		for (int i=0;i<nl.size();i++) {
			Node node=(Node)nl.get(i);
			pckClassName=NameParser.getLongClassName(node);
			if (!pckClassName.equals(prevPckClass)) {
				// this is a new name set package/class relation
				lpn=NameParser.getLongPckageName(node);
				lcn=NameParser.getLongClassName(node);
				if (!lpn.equals("") && !lcn.equals("")) { // no bad data
					// set package/class
					LogicalPackage lp;
					lp=(LogicalPackage)pck.get(lpn);
					if (lp==null) {
						lp=new LogicalPackage(trace.getScenario().getSystem(),lpn);
						pck.put(lpn,lp); // cache package
					}
					ClassElement ce=((MethodElement)node.getBehaviouralElement()).getClassElement();
					ce.setPackage(lp);
					cls.put(lcn,ce);  // cache class
					classElements.put(ce.getName(),ce); // name as the key,add for later process
				}
			}
			prevPckClass=pckClassName;
		}
		
		ArrayList pl=trace.getState().getPackageList();
		pl.addAll(pck.values());
		ArrayList cl=trace.getState().getClassList();
		cl.addAll(cls.values());				
		//At the end of this method, classElements hashtable is filled for setClassHierarchy() method
	}	

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setClassHierarchy()
	 */
	public void setClassHierarchy() {
		ClassLoader urlLoader = new URLClassLoader(getURL()); // loader
		Enumeration en=classElements.elements();
		while (en.hasMoreElements()) {
			ClassElement ce=(ClassElement)en.nextElement();
			try {
				Class currentClass = Class.forName(ce.getName(), true, urlLoader);
				Class superClass = currentClass.getSuperclass();
				if (superClass != null
					&& !superClass.getName().startsWith("java.")
					&& !superClass.getName().startsWith("javax.")
					&& !superClass.getName().startsWith("org.")) {
				
					// find super from hash, or can be : system.getModelElement
					ClassElement superClassElem =(ClassElement)classElements.get(superClass.getName());
					superClassElem.addSubclass(ce);
					ce.addSuperclass(superClassElem);
					}	
			
				// Also processing with setClassIsInterface functionalities 
				ce.setInterfaceFlag(currentClass.isInterface());
			
				// Also sets MethodElement attributes if it's included in the method lsit;
				// as required in setMethodAttribute() 
				setClassMethods(currentClass,ce);
				
				//counter
				numberOfParsedClasses++;
			} catch (ClassNotFoundException cnfe) {
				System.out.println("JavaClassParser: Cannot load class- "+ce.getName());
				errNotParsedClasses.add(ce.getName());
			}
		}		
	}
	
	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setClassIsInterface()
	 */
	public void setClassIsInterface() {
		// TODO Auto-generated method stub
		// Do nothing here , work is done in setClassHierarchy(), line: 
		// ce.setInterfaceFlag(currentClass.isInterface());

	}	
	
	/* (non-Javadoc)
	 * @see seat.ui.algorithm.IParsing#setMethodAttribute()
	 */
	public void setMethodAttribute() {
		//Note: most method attributes are already set in setClassHierarchy()
		// when class is loaded. Here only set thos constructors <init> 
		ArrayList nl=trace.getState().getModelNodeList();
		for (int i=0;i<nl.size();i++) {
			Node node=(Node)nl.get(i);
			if (node.getLabel().endsWith("<init>"))
				((MethodElement)node.getBehaviouralElement()).setConstructorFlag(true);
		}
	}


	/**
	 * Finds all the methods of the givwn class and sets attributes of those methods
	 * that exist in trace node list.
	 */
	private void setClassMethods(Class currentClass,ClassElement classElem) {
		String fullClassName=currentClass.getName();
		Method[] methods = currentClass.getDeclaredMethods();
		numberOfMethods+=methods.length;

		for (int j = 0; j < methods.length; j++) {
			MethodElement methElem = findMethodElement(fullClassName+"."+methods[j].getName());// if used in trace
			if (methElem==null) // this method is not called in trace
				continue;
			// Set the method modifier
			int mod = methods[j].getModifiers();
			methElem.PUBLIC = Modifier.isPublic(mod);
			methElem.PRIVATE = Modifier.isPrivate(mod);
			methElem.PROTECTED = Modifier.isProtected(mod);
			methElem.STATIC = Modifier.isStatic(mod);
			methElem.ABSTRACT = Modifier.isAbstract(mod);
			methElem.FINAL= Modifier.isFinal(mod);
		}
	}

	/**
	 * Gets URL for active and source projects 
	 * @param ce- its name is used
	 * @return null if not find
	 */
	private URL[] getURL() {
		// list to save url
		this.urlList=new ArrayList(); 
		findPotentialURL(activeProject.getLocation().toFile());
		for (int i=0;i<sourceProjects.length;i++) 
			findPotentialURL(sourceProjects[i].getLocation().toFile());
		return (URL[])urlList.toArray(new URL[urlList.size()]);
	}
	
	/**
	 * Searches for directory or files that can be a url and adds them to the gobal vector.
	 * A directory and .jar file is considered a candidatye of a url.
	 * @param dir
	 */
	private void findPotentialURL(File dir) {
		String[] files = dir.list();  // files contains the list of files/dirs of dir  	
		// Browse the directory 
		File currentFile=dir;
		try {
			if (currentFile.isDirectory()) 
				urlList.add(currentFile.toURL());			
			for (int i = 0; i < files.length; i++) {  
			   currentFile = new File(dir, files[i]);
			   if (currentFile.isFile()) {
			   		if (currentFile.getName().endsWith(".jar"))
						urlList.add(currentFile.toURL());
			   } else 
				   findPotentialURL(currentFile);
			}
		} catch (MalformedURLException mue) {
			System.out.println("JavaClassParser: Cannot convert path to URL- "+currentFile.getName());
		}
	}
//	private File findDirectory(File dir,ClassElement ce) {
//		String[] files = dir.list();  // files contains the list of files/dirs of dir  	
//		// Browse the directory 
//		for (int i = 0; i < files.length; i++) {  
//		   File currentFile = new File(dir, files[i]);
//		   if (currentFile.isFile()) {
//			  if (currentFile.getName().endsWith(fileExtension)&&(currentFile.getName().equals(ce.getName())))
//				return currentFile;
//		   } else
//			   findDirectory(dir,ce);
//		}
//		return null;	 
//	}
	/**
	 * Goes through the node list in trace and find if the MethodElement exists by the given name.
	 * Returns null if not found which means this method is not used in trace
	 */
	private MethodElement findMethodElement(String fullyQualfiedName) {
		ArrayList nl=trace.getState().getModelNodeList();
		for (int i=0;i<nl.size();i++) {
			Node node=(Node)nl.get(i);
			if (node.getLabel().equals(fullyQualfiedName))
				return (MethodElement)node.getBehaviouralElement();
		}
		return null;
	}

}