package seat.ui.algorithm;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IParent;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.model.core.structure.ClassElement;
import seat.model.core.structure.LogicalPackage;
import seat.model.core.structure.MethodElement;
import seat.model.core.structure.SoftwareSystem;

/**
 * The JavaParser uses JDT core to parse required java classes for trace.
 * The current parser relies on where source projects are located and associated
 * with current trace. This impl of JavaParser uses project list saved in SeatExtra
 * class which temporarily in SeatPlugin.
 */
public class JavaParser extends Parser {
	private Hashtable classElements=new Hashtable(); // used to establish inheritance relation
	private Hashtable javaClasses;  // for JavaElements from java core parsing
	
	
	public JavaParser(Trace trace,IProject active, IProject[] sourcePrj) {
		super(trace,active,sourcePrj);
		// TODO Auto-generated constructor stub
		javaClasses=trace.getState().getJavaClasses();
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setPackageClassRelation()
	 */
	public void setPackageClassRelation() {
		// TODO Auto-generated method stub
		Hashtable pck=new Hashtable();
		Hashtable cls=new Hashtable();

		ArrayList nl=trace.getState().getModelNodeList();
		String prevPckClass="";
		String pckClassName,lpn,lcn;
		int lastDot=-1;
		for (int i=0;i<nl.size();i++) {
			Node node=(Node)nl.get(i);
			pckClassName=NameParser.getLongClassName(node);
			if (!pckClassName.equals(prevPckClass)) {
				// this is a new name set package/class relation
				lpn=NameParser.getLongPckageName(node);
				lcn=NameParser.getLongClassName(node);
				if (!lpn.equals("") && !lcn.equals("")) { // no bad data
					// set package/class
					LogicalPackage lp;
					lp=(LogicalPackage)pck.get(lpn);
					if (lp==null) {
						lp=new LogicalPackage(trace.getScenario().getSystem(),lpn);
						pck.put(lpn,lp); // cache package
					}
					ClassElement ce=((MethodElement)node.getBehaviouralElement()).getClassElement();
					ce.setPackage(lp);
					cls.put(lcn,ce);  // cache class
					classElements.put(ce.getName(),ce); // name as the key,add for later process
				}
			}
			prevPckClass=pckClassName;
		}
		
		ArrayList pl=trace.getState().getPackageList();
		pl.addAll(pck.values());
		ArrayList cl=trace.getState().getClassList();
		cl.addAll(cls.values());				
		//At the end of this method, classElements hashtable is filled for setClassHierarchy() method
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setClassHierarchy()
	 */
	public void setClassHierarchy() {
		// TODO Auto-generated method stub
		String extension="java";
		Enumeration en=classElements.elements();
		while (en.hasMoreElements()) {
			ClassElement ce=(ClassElement)en.nextElement();
			IJavaElement element=this.createClassFromName(ce.getName(),extension);
			if ( (element != null) && (element instanceof IMember)) {
				javaClasses.put(ce.getName(),element); // save for later setMethodAttributes
				ICompilationUnit cu= ((IMember) element).getCompilationUnit();
				if (cu != null && cu.isWorkingCopy()) {
					element= cu.getOriginal(element);
					if (element.getElementType() == IJavaElement.TYPE) {
						IType type= (IType) element;
						try {
							ce.setInterfaceFlag(type.isInterface());
							ITypeHierarchy fHierarchy= type.newTypeHierarchy(new NullProgressMonitor());
							IType[] supers=fHierarchy.getAllSuperclasses(type);
							for (int k=0;k<supers.length;k++) {
								ClassElement sup=(ClassElement)classElements.get(supers[k].getFullyQualifiedName());
								if (sup!=null) // some may not be parsed, like Object, ignore them
									ce.addSuperclass(sup);
							}
						} catch (JavaModelException jme) {}
					}
				}
			}						
		} // while
		//At the end of this method, javaClasses hashtable is filled for setMethodsAttribute() method
	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.Parser#setClassIsInterface()
	 */
	public void setClassIsInterface() {
		// TODO Auto-generated method stub
		// Do nothing here , work is done in setClassHierarchy(), line: 
		// ce[i].setInterfaceFlag(type.isInterface());

	}

	/* (non-Javadoc)
	 * @see seat.ui.algorithm.IParsing#setMethodAttribute()
	 */
	public void setMethodAttribute() {
		// TODO Auto-generated method stub
		ArrayList nl=trace.getState().getModelNodeList();
		Node node=null;
		String longClassName=null,methodName;
		IParent je=null;
		for (int i=0;i<nl.size();i++) {
			node=(Node)nl.get(i);
			longClassName=NameParser.getLongClassName(node);
			methodName=NameParser.getShortMethodName(node);
			
			// the following code converts <init> to constructor-class name
			if (methodName.equalsIgnoreCase("<init>"))
				methodName=NameParser.getShortClassName(node);
				
			je=(IParent)javaClasses.get(longClassName);
			if (je==null) // in case error
				continue;
			try { 
				IJavaElement[] children=je.getChildren();
				for (int k=0;k<children.length;k++) {
					IJavaElement child=children[k];
					if (child.getElementName().equals(methodName) &&(child.getElementType()==IJavaElement.METHOD) ) {
						IMethod method=(IMethod)child;
						int flag=method.getFlags();
						MethodElement me=(MethodElement)node.getBehaviouralElement();
						me.ABSTRACT=Flags.isAbstract(flag);
						me.FINAL=Flags.isFinal(flag);
						me.PRIVATE=Flags.isPrivate(flag);
						me.PROTECTED=Flags.isProtected(flag);
						me.PUBLIC=Flags.isPublic(flag);
						me.STATIC=Flags.isStatic(flag);
						me.setConstructorFlag(((IMethod)child).isConstructor());
						me.setDescription(JavaDocAccess.getJavaDocTextString(method,true));
					}
				}
			} catch (JavaModelException jme) {
				SeatUtil.showMessage("Error occurredd when parsing Jva source code!");
			}
		}	
			
	}

	/**
	 * This method is not from Parser framework method. It can be called just after a javaParser
	 * is instanitated to parse all needed Classes for succeeding use. Such as static method:
	 * getClassComment();
	 */
	public void parseClassSource() {
		if (javaClasses.isEmpty()) { // parse needed!
			// find all classes need to be parsed, some classes may be unnecessary for this trace
			String extension="java";
			SoftwareSystem system=trace.getScenario().getSystem();
			Enumeration en=system.getModelElements();
			while(en.hasMoreElements()) {
				Object obj=en.nextElement();
				if (obj instanceof ClassElement) {
					String className=((ClassElement)obj).getName();
					IJavaElement element=createClassFromName(className,extension);
					if ( (element != null) && (element instanceof IMember)) 
						javaClasses.put(className,element); // save for later setMethodAttributes					
				}
			}
		}			
	}

	/**
	 * Gets class comments that is normally before the class definition. 
	 * @param javaClasses
	 * @param longClassName
	 * @return
	 */
	public static String getClassComment(Trace trace, String longClassName) {
		Hashtable javaClasses=trace.getState().getJavaClasses();
		IJavaElement je=(IJavaElement)javaClasses.get(longClassName);
		if ( (je!=null)&&(je instanceof IMember)) { // in case error
			try { 
				return JavaDocAccess.getJavaDocTextString((IMember)je,true);
			} catch (JavaModelException jme) {
				SeatUtil.showMessage("Error occurredd when parsing Jva source code!");
				return "";
			}			
		} else
			return ""; 
	}	

	/**
	 *  Gets comments for a node
	 */
	public static String getMethodComment(Trace trace,Node node) {
		Hashtable javaClasses=trace.getState().getJavaClasses();
		String longClassName=NameParser.getLongClassName(node);
		String methodName=NameParser.getShortMethodName(node);
		IParent je=(IParent)javaClasses.get(longClassName);
		if (je!=null) { // in case error
			try { 
				IJavaElement[] children=je.getChildren();
				for (int k=0;k<children.length;k++) {
					IJavaElement child=children[k];
					if (child.getElementName().equals(methodName) &&(child.getElementType()==IJavaElement.METHOD) ) {
						IMethod method=((IMethod)child);
						return JavaDocAccess.getJavaDocTextString(method,true);
					}
				}
			} catch (JavaModelException jme) {
				SeatUtil.showMessage("Error occurredd when parsing Jva source code!");
			}
		}
		return ""; 
	}


	/**
	 * Creates the IJaveElement from the given class name.
	 * ClassName includes packge.class, e,g, org.eclipse.ui.Control.
	 * @param fullClassName as above,
	 * @param extension, here is 'java', not dot before java
	 * @return null if no JavaElement can be created(no source)
	 */
	private IJavaElement createClassFromName(String pckClassName,String extension) {
		String pn=pckClassName.replace('.','/');
		Path p=new Path(pn);
		p=(Path)p.addFileExtension("java");

		IWorkspaceRoot root=ResourcesPlugin.getWorkspace().getRoot();
		IResource project=root.findMember(activeProject.getFullPath());
		IFile file=((IContainer)project).getFile(p);
		boolean found =false;
		if (file.exists())
			found =true;
		else { // look into project list
			for (int i=0;i<sourceProjects.length;i++) {
//				if (sourceProjects[i].getNature()) // use proejct nature to test later on
				project=root.findMember(sourceProjects[i].getFullPath());
				file=((IContainer)project).getFile(p);
				if (file.exists()) {
					found=true;
					break;				
				}
			}				
		}
		if (!found)
			return null;

		// creates and get SourceType from file: top level element
		try {
			ICompilationUnit unit=JavaCore.createCompilationUnitFrom(file);
			IJavaElement fInput=(IJavaElement)unit.getWorkingCopy();
			if (fInput instanceof IParent) {
				IParent classFile= (IParent) fInput;
				IJavaElement[] javaElements=classFile.getChildren();
				String className=NameParser.getShortClassName(pckClassName);
				for (int j=0;j<javaElements.length;j++)
					if (javaElements[j].getElementName().equals(className)) 
						return javaElements[j];
			}
		} catch(JavaModelException jme) {}
		return null;						
	}
		
}