package seat.ui.algorithm;

/**
 * A replication of how internal hidden scheme works. 
 * It is used for conetxt menu to process a subtree, and
 * these subtree are cached in Exploration class.
 */
public class NodeBits {
	private int algorithmBits = 0;

	/**
	 * Constructs and applies an algorithm.
	 * ai is the unique index of algorithm  
	 */
	public NodeBits(int ai) {
		apply(ai);
	}

	public NodeBits() {
	}

	/**
	 * ai start from 0 till (maximum index number -1)
	 */
	public void apply(int ai) {
		this.algorithmBits |= (1 << ai);
	}

	/** Shows this node. 
	 * This node and the subtree rooted at it are shown. 
	 */
	public void revoke(int ai) {
		this.algorithmBits &= ~(1 << ai);
	}

	/**
	 * Resets to 0 to show it. Clears all bits, used only in TreeTrace.reconstitute()
	 */
	public void revokeAll() {
		this.algorithmBits = 0;
	}

	/**
	 * Test if algoritm is applied.
	 * @param ai
	 * @return
	 */
	public boolean isApplied(int ai) {
		int b = this.algorithmBits & (1 << ai);
		return (b != 0) ? true : false;
	}
}
