package seat.ui.preference;

import java.util.StringTokenizer;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.FontFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PathEditor;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import seat.SeatPlugin;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.Parameter;

/**
 * Created on Jul 17, 2003
 * 
 * Parameter convention for an algorithms:
 * 1. The developer of an algorithm specify all global unique id for each parameter, defined in plugin.xml
 * 2. Inside the init() of his algorithm, he will retrieve the parameter by above id,
 * 3. Parameters are save in the platform by their IDs(no extra info added). 
 */
public class AlgorithmPreference
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

private static AlgorithmDescription[] algs;
private int iAlgorithm;
/**
 * A field editor for displaying labels not associated with other widgets.
 */
class LabelFieldEditor extends FieldEditor {

	private Label label;

	// All labels can use the same preference name since they don't
	// store any preference.
	public LabelFieldEditor(String value, Composite parent) {
		super("label", value, parent);
	}

	// Adjusts the field editor to be displayed correctly
	// for the given number of columns.
	protected void adjustForNumColumns(int numColumns) {
		((GridData) label.getLayoutData()).horizontalSpan = numColumns;
	}

	// Fills the field editor's controls into the given parent.
	protected void doFillIntoGrid(Composite parent, int numColumns) {
		label = getLabelControl(parent);
		
		GridData gridData = new GridData();
		gridData.horizontalSpan = numColumns;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = false;
		gridData.verticalAlignment = GridData.CENTER;
		gridData.grabExcessVerticalSpace = false;
		
		label.setLayoutData(gridData);
	}

	// Returns the number of controls in the field editor.
	public int getNumberOfControls() {
		return 1;
	}

	// Labels do not persist any preferences, so these methods are empty.
	protected void doLoad() {
	}
	protected void doLoadDefault() {
	}
	protected void doStore() {
	}
}


	/**
	 * @param arg0
	 */
	public AlgorithmPreference() {
		// TODO Auto-generated constructor stub
		super(FieldEditorPreferencePage.GRID);
		algs=SeatPlugin.getAlgorithmDescriptions();
		setPreferenceStore(SeatPlugin.getDefault().getPreferenceStore());
		setDefaultConfig(getPreferenceStore());		
//		setDescription("efu");
	}
	
	protected void createFieldEditors() {
		// TODO Auto-generated method stub
		addField(new LabelFieldEditor(algs[iAlgorithm].getDescription(),getFieldEditorParent()));					
		
		//getFieldEditorParent() must not be cached
//		int i=iAlgorithm;
		Parameter[] paras=algs[iAlgorithm].getParameters();
		String algIndex=algs[iAlgorithm].getIndex()+"@#@"; // index will added as prefix to property id 
		// and it will be used by propertychangelisteners to identify which algoirthm need to be reapplied.
		if (paras!=null) {			
			for (int j=0;j<paras.length;j++) {
				if (paras[j].getType().equalsIgnoreCase("Boolean")) {
					addField(new BooleanFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));					
				}
				else if (paras[j].getType().equalsIgnoreCase("Integer")) 
					addField(new IntegerFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));
				else if (paras[j].getType().equalsIgnoreCase("String")) 
					addField(new StringFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));					
				else if (paras[j].getType().equalsIgnoreCase("Color")) 
					addField(new ColorFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));					
				else if (paras[j].getType().equalsIgnoreCase("Directory")) 
					addField(new DirectoryFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));
				else if (paras[j].getType().equalsIgnoreCase("File")) 
					addField(new FileFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));
				else if (paras[j].getType().equalsIgnoreCase("Path")) 
					addField(new PathEditor(algIndex+paras[j].getID(),paras[j].getName(),"Please chhose a path",getFieldEditorParent()));
				else if (paras[j].getType().equalsIgnoreCase("Font")) 
					addField(new FontFieldEditor(algIndex+paras[j].getID(),paras[j].getName(),getFieldEditorParent()));
			}
		}
	  }

	// get the index of algorithm by dialog title,-1 if not found
	// but because getTitle() can not be used in Constructor and init(),
	// therefore put setDescription/ loaddefaultconfig to the begining of createFieldEditors
	private int getAlgIndexByName() {
		String dialogTitle=this.getTitle();
		int index=-1;
		for (int i=0;i<algs.length;i++)
			if (algs[i].getName().equals(dialogTitle))
				return i;
		return -1;	
	}
	
	private String formatString(String desc) {
		StringTokenizer splits= new StringTokenizer(desc);
		StringBuffer wellform=new StringBuffer("");
		while (splits.hasMoreTokens()) {
			wellform.append(splits.nextToken()+" ");
		}
		return wellform.toString();
	}
	
	private static void setDefaultConfig(IPreferenceStore store,int index) {
		String algIndex=algs[index].getIndex()+"@#@"; // index will added as prefix to property id 
		Parameter[] paras=algs[index].getParameters();
		if (paras!=null) {			
			for (int j=0;j<paras.length;j++) {
				if (paras[j].getType().equals("Boolean"))
					store.setDefault(algIndex+paras[j].getID(),paras[j].getDefaultData().equalsIgnoreCase("true")?true:false); 
				else if (paras[j].getType().equals("Integer")) 
					try {
						store.setDefault(algIndex+paras[j].getID(),Integer.parseInt(paras[j].getDefaultData()));
					} catch (NumberFormatException nfe) {
						store.setDefault(algIndex+paras[j].getID(),0);
					} 
				else if (paras[j].getType().equals("String")) 
					store.setDefault(algIndex+paras[j].getID(),paras[j].getDefaultData());
				else if (paras[j].getType().equals("Color")) 
					PreferenceConverter.setDefault(store,algIndex+paras[j].getID(),PreferenceConverter.COLOR_DEFAULT_DEFAULT);
				else if (paras[j].getType().equals("Directory")) 
					store.setDefault(algIndex+paras[j].getID(),paras[j].getDefaultData());
				else if (paras[j].getType().equals("File")) 
					store.setDefault(algIndex+paras[j].getID(),paras[j].getDefaultData());
				else if (paras[j].getType().equals("Path")) 
					store.setDefault(algIndex+paras[j].getID(),paras[j].getDefaultData());
				else if (paras[j].getType().equals("Font")) 
					PreferenceConverter.setDefault(store,algIndex+paras[j].getID(),PreferenceConverter.FONTDATA_DEFAULT_DEFAULT);
			}
		}
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		// TODO Auto-generated method stub
		// move the setPreferenceStore(TreeViewerPlugin..) to the constructor
		int i=0;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		// TODO Auto-generated method stub
		iAlgorithm=getAlgIndexByName();
		if (iAlgorithm!=-1) {
//			GridData gd = new GridData();
////						gd.heightHint=100;
//			gd.horizontalAlignment = GridData.FILL;
//			gd.verticalAlignment= GridData.FILL;
//			gd.grabExcessVerticalSpace=true;
//			Text des=new Text(parent,SWT.MULTI|SWT.WRAP);//this.createDescriptionLabel(parent);
//			des.setLayoutData(gd);
//			des.setText(formatString(algs[iAlgorithm].getDescription()));

			String ddd=getDescription();
			// data was set,but not refreshed
			this.setDescription(algs[iAlgorithm].getDescription());
			

			setDefaultConfig(getPreferenceStore(),iAlgorithm);	
			this.setVisible(true);
		}				
		return super.createContents(parent);
	}

	public static void setDefaultConfig(IPreferenceStore store) {
		algs=SeatPlugin.getAlgorithmDescriptions();
		for (int i=0;i<algs.length;i++)
			setDefaultConfig(store,i);
	}
}
