package seat.ui.preference;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.TableItem;

import seat.ui.preference.TableFieldEditor.IconConfig;

/**
 * This class implements an ICellModifier
 * An ICellModifier is called when the user modifes a cell in the 
 * tableViewer
 */

public class TableCellModifier implements ICellModifier {
	private TableViewer tv; 
	private String[] columnNames;
	
	/**
	 * Constructor 
	 * @param TableViewerExample an instance of a TableViewerExample 
	 */
	public TableCellModifier(TableViewer tv,String[] columnNames) {
		super();
		this.tv=tv;
		this.columnNames=columnNames;
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
	 */
	public boolean canModify(Object element, String property) {
		int columnIndex = findIndex(property);		
		return (columnIndex==0)?false:true;
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
	 */
	public Object getValue(Object element, String property) {

		// Find the index of the column
		int columnIndex = findIndex(property);
		IconConfig row=(IconConfig)element;		
		Object result = null;

		switch (columnIndex) {
			case 0 : 
				result = row.name;
				break;
			case 1 : // lower left corner
				result = row.icon1;
				break;
			case 2 : //  lower right
				result = row.icon2;
				break;
			case 3 : //  
				result = row.icon3;
				break;
			case 4 : //  
				result = row.icon4;
				break;
			default :
				result = "";
		}
		return result;	
	}

	/**
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {	

		// Find the index of the column 
		int columnIndex	= findIndex(property);
			
		TableItem item = (TableItem) element;
		IconConfig ar= (IconConfig) item.getData();
		String valueString;

		switch (columnIndex) {
			case 0 : // COMPLETED_COLUMN 
				break;
			case 1 : // DESCRIPTION_COLUMN
				ar.icon1=(Boolean) value;
				if (((Boolean) value).booleanValue()) {
					ar.icon2=new Boolean(false);
					ar.icon3=new Boolean(false);
					ar.icon4=new Boolean(false);
				}
				break;
			case 2 : // OWNER_COLUMN 
				ar.icon2=(Boolean) value;
				if (((Boolean) value).booleanValue()) {
					ar.icon1=new Boolean(false);
					ar.icon3=new Boolean(false);
					ar.icon4=new Boolean(false);
				}
				break;
			case 3 : // PERCENT_COLUMN
				ar.icon3=(Boolean) value;
				if (((Boolean) value).booleanValue()) {
					ar.icon1=new Boolean(false);
					ar.icon2=new Boolean(false);
					ar.icon4=new Boolean(false);
				}
				break;
			case 4 : // PERCENT_COLUMN
				ar.icon4=(Boolean) value;
				if (((Boolean) value).booleanValue()) {
					ar.icon1=new Boolean(false);
					ar.icon2=new Boolean(false);
					ar.icon3=new Boolean(false);
				}
				break;
			default :
				break;
			}
		tv.update(ar,null);
	}
	
	private int findIndex(String property) {
		for (int i=0;i<columnNames.length;i++)
			if (columnNames[i].equals(property))
				return i;
		return 0;
	}

}
