package seat.ui.tree.partial;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.util.Assert;
import org.eclipse.search.ui.ISearchPageScoreComputer;
import org.eclipse.ui.views.properties.IPropertySource;

import seat.SeatUtil;
import seat.model.algorithms.Criteria;
import seat.model.core.dynamic.Node;
import seat.model.core.dynamic.Trace;
import seat.ui.views.MethodPropertySource;
import seat.ui.views.TraceSearchPageScore;

/**
 * Implentation base for a path. This class mainly contructs parent/children relation
 * and other related methds.
 */
public class TreeNode implements IAdaptable {
	private Trace trace;  //remove static, interfere btwn 2 editors

	protected Node node; // node from model
	protected TreeNode parent=null; 
	protected ArrayList children=new ArrayList();
	private String imageID=SeatUtil.ICON_METHOD;
	private int color=0;
//	private boolean hidden=false;
	private int lineNo;
	/**
	 * 
	 */
	public TreeNode(Node node) {
		super();
		// TODO Auto-generated constructor stub
		this.node=node;
	}

	public TreeNode() {
		super();
		// TODO Auto-generated constructor stub
		this.node=null; // must use setNode() to ensure not null
	}

	private MethodPropertySource meps; // for property display
	private ISearchPageScoreComputer spsc; // score for search
	/** 
	* Class adapter
	* @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	*/
	public Object getAdapter(Class adapter) {
		// TODO Auto-generated method stub
		if (adapter==IPropertySource.class) {
			if (meps==null) {
				// cache
				meps=new MethodPropertySource(this);
			}
			return meps;
		} else if (adapter==ISearchPageScoreComputer.class) {
			if (spsc==null) {
				spsc=new TraceSearchPageScore();
			}
			return spsc;
		}
		return null;
		}
	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#getParent()
	 */
	public TreeNode getParent() {
		// TODO Auto-generated method stub
		return parent;
	}

	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#getChildren()
	 */
	public TreeNode[] getChildren() {
		// TODO Auto-generated method stub
		return (TreeNode[])children.toArray(new TreeNode[children.size()]);
	}

	/**
	 * Tests and returns if a node has any unhidden children.
	 * A node must have getChildAt/isHidden method. 
	 * Only unhidden nodes are returned.
	 * @return
	 */
	public boolean hasUnhiddenChildren() {
		// TODO Auto-generated method stub
		Assert.isNotNull(node);
		Node child;		
		int length=node.getNumberOfChildren();
		for (int i=0;i<length;i++) {
			child=node.getChildAt(i);
			if (!child.isHidden())  // hidden child is not counted 
				return true; 
		}
		return false; // no child found yet to this point
	}

	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#addChild(seat.ui.viewer.TreeNode)
	 */
	public void addChild(TreeNode child) {
		// TODO Auto-generated method stub
		Assert.isNotNull(child);
		children.add(child);
	}

	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#setParent(seat.ui.viewer.TreeNode)
	 */
	public void setParent(TreeNode par) {
		// TODO Auto-generated method stub
		// Note: parent can be non-existent(null), so not use: Assert.isNotNull(parent); 
		parent=par;
	}
	
	/**
	 * Gets sibling from parent/children link. 
	 * @return
	 */
	public TreeNode getNextSibling() {
		TreeNode parent=this.getParent();
		if (parent==null)
			return null;
		TreeNode[] children=parent.getChildren();
		boolean found=false;
		int foundId=-1;
		for (int i=0;i<children.length;i++)
			if (children[i]==this) {
				found=true;
				foundId=i;
				break;
			}
		if (found&&foundId<children.length-1)
			return children[foundId+1];
		else 
			return null;
	}

	public TreeNode getPrevSibling() {
		TreeNode parent=this.getParent();
		if (parent==null)
			return null;
		TreeNode[] children=parent.getChildren();
		boolean found=false;
		int foundId=-1;
		for (int i=0;i<children.length;i++)
			if (children[i]==this) {
				found=true;
				foundId=i;
				break;
			}
		if (found&&foundId!=0)
			return children[foundId-1];
		else 
			return null;
	}
	
	

	public int getRepetition() {
		// TODO Auto-generated method stub
		// Note: parent can be non-existent(null), so not use: Assert.isNotNull(parent);
		int basic=1;
		if (Criteria.ignoreRepetitions)
			return basic;
		if (parent!=null) {
			Node np=parent.getElement(); // find edge index, rep info is inside edge
			Node nc=this.node;
			for (int i=0;i<np.getNumberOfChildren();i++)
				 if (np.getChildAt(i)==nc) {
					basic=np.getEdge(i).getNumberOfRepetitions();
					break;
				 }
		}
		return basic+repModifier; 
	}
	
	private int repModifier=0;
	public void addRepModifier(int modi) {
		if (modi==0)
			repModifier=0;
		else
			repModifier+=modi;
	}

	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#setTrace(seat.model.core.dynamic.Trace)
	 */
	public void setTrace(Trace trace) {
		this.trace=trace;
	}

	public Trace getTrace() {
		Assert.isNotNull(trace);
		return this.trace;
	}
	public void setElement(Node nod) {
		Assert.isNotNull(nod);
		this.node=nod;
	}
	public Node getElement() {
		Assert.isNotNull(node);
		return node;
	}
	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#getImageID()
	 */
	public String getImageID() {
		// TODO Auto-generated method stub
		return imageID;
	}

	/* (non-Javadoc)
	 * @see seat.ui.viewer.TreeNode#setImageID(int)
	 */
	public void setImageID(String id) {
		// TODO Auto-generated method stub
		imageID=id;
	}

//	public abstract String getName();
//	public abstract String getFullQualifiedName();
//	public abstract String getType();
//	public abstract String getScope();
//	public abstract int    getNumOfChildren();
//	public abstract int    getLevelOfCall();
//	public abstract String getDescription();	
//	public abstract void   setDescription(String desc);	

	public String getName() {
		return node.getLabel();
	}
	public String getFullQualifiedName() {
		return node.getLabel();		
	}
	public String getType() {
		String type="to be decided";
		return type;
	}
	public String getScope() {
		String scope="to be decided";
		return scope;
	}
	
	public int getNumOfUnhiddenChildren() { // ONLY count visible
		int counter=0;
		for (int i=0;i<node.getNumberOfChildren();i++) {
			 if (!node.getChildAt(i).isHidden())
				counter++;
		}
		return counter;
	}

	public int getLevelOfCall() {		
		return (parent==null)?1:parent.getLevelOfCall()+1;	
	}
	
	String description="";
	
	public String getDescription() {
		return description;
	}
	public void setDescription(String d) {
		description=d;
	}	


	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#getColored()
	 */
	public int getColor() {
		// TODO Auto-generated method stub
		return color;
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#setColored(boolean)
	 */
	public void setColor(int colorID) {
		// TODO Auto-generated method stub
		this.color=colorID;
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#inside(java.util.List)
	 */
	public boolean inside(List list) {
		// TODO Auto-generated method stub
		return list.contains(this.getElement());
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#clearChildren()
	 */
	public void clearChildren() {
		// TODO Auto-generated method stub
		children.clear();
	}

//	/* (non-Javadoc)
//	 * @see seat.ui.tree.TreeNode#getHidden()
//	 */
//	public boolean isHidden() {
//		// TODO Auto-generated method stub
//		return hidden;
//	}
//
//	/* (non-Javadoc)
//	 * @see seat.ui.tree.TreeNode#setHidden(boolean)
//	 */
//	public void setHidden(boolean hidden) {
//		// TODO Auto-generated method stub
//		this.hidden=hidden;
//	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#setNode(seat.model.core.dynamic.Node)
	 */
	public void setNode(Node node) {
		// TODO Auto-generated method stub
		this.node=node;
	}

	/**
	 * @return
	 */
	public int getLineNo() {
		return lineNo;
	}

	/**
	 * @param i
	 */
	public void setLineNo(int i) {
		lineNo = i;
	}

	/* (non-Javadoc)
	 * @see seat.ui.tree.TreeNode#getModelIndexOfParent()
	 */
	public int getModelIndexOfParent() {
		// TODO Auto-generated method stub
		return ((TreeNode)parent).children.indexOf(this);
	}

}
