/*
 * Created on Jun 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.tree.viewer;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.io.ITraceSource;
import seat.model.core.dynamic.MethodInvocationTrace;
import seat.model.core.dynamic.Scenario;
import seat.model.core.dynamic.Trace;
import seat.model.core.structure.SoftwareSystem;
import seat.model.core.structure.StreamSource;
import seat.ui.algorithm.GlobalUtilityFilter;
import seat.ui.algorithm.JavaClassParser;
import seat.ui.algorithm.JavaParser;
import seat.ui.algorithm.Parser;
import seat.ui.views.MethodView;
import seat.ui.views.ModelView;
import seat.ui.views.PatternView;
import seat.ui.views.UtilityView;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class MultiTraceEditor extends MultiPageEditorPart implements IPartListener{
	private TraceEditorPage defEditor, extraEditor; // editor page
	private InputStream traceStream;
	private IFile traceFile;
	private Trace trace;
	private int ascentedPageIndex=0; 				// largest number for index
	private int previousPage;						// previous page index
	private MultiTraceEditor previousEditor;		// previous active, used in PartListener
	
	// auxillary views, will be set when view starts
	private ModelView  modelView;
	private UtilityView utilView;					
	private PatternView patternView;
	protected IProject activeProject;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.MultiPageEditorPart#createPages()
	 */
	protected void createPages() {
		// TODO Auto-generated method stub
		long startTime=0,endTime=0;
		startTime = System.currentTimeMillis();
			
		trace=loadTrace(); // must not be put in cosntructor, need prj infor from init

		defEditor = new TraceEditorPage(this,new Exploration(trace),true);
		addNewExploration(defEditor);  // default editor
		setActivePage(0);
		this.setTitle(trace.getTraceSource().getName());
		
		endTime = System.currentTimeMillis();
		System.out.println("Total open time(load trace, parse, open editor): " + (endTime - startTime)/1000.0 +"sec");
	}

	/**
	 * Creates a new editor for extra explorations besides the default one.
	 *
	 */
	public void addNewExploration() {
		extraEditor = new TraceEditorPage(this,new Exploration(trace),false);
		addNewExploration(extraEditor);
	}
	
	/**
	 * Deletes an editor page and closes the exploration associated with it.
	 * Note: the first(default)page can not be closed.
	 */
	public void deleteExploration() {
		int act=getActivePage();
		setActivePage(0);
		if (act!=-1)
			removePage(act);
		pageChange(getActivePage());
	}

	/*
	 * Adds a new exploration editor page, either default or extra.
	 */ 
	private void addNewExploration(TraceEditorPage editor) {
		try {
			int index= addPage(editor, getEditorInput());
			setPageText(index, "Exploration "+ascentedPageIndex);
			setActivePage(index);
			pageChange(getActivePage());
			ascentedPageIndex++;				
		} catch (PartInitException e) {
			ErrorDialog.openError(getSite().getShell(), "Cannot create multiple editors", null, e.getStatus()); //$NON-NLS-1$
		}
	}
	
//	/*
//	 * Note: This method must be called if there is a page change incurred by
//	 * method call, such as setActivePage() as in addNewExploration and deleteExploration.	
//	 * Note: This method will cause a setActivePage(IEditorPart activeEditor) event in 
//	 * MultiPageEditorActionBarContributor, and that method will synchronize action bar.	
//	 * (non-Javadoc)
//	 * @see org.eclipse.ui.part.MultiPageEditorPart#pageChange(int)
//	 */
//	protected void pageChange(int newPageIndex) {
//		// TODO Auto-generated method stub
//		super.pageChange(newPageIndex);	
//		if (newPageIndex==-1) // bad index
//			return;	
//		TraceEditorPage prev=(TraceEditorPage)getEditor(previousPage);
//		TraceEditorPage curr=(TraceEditorPage)getEditor(newPageIndex);
//		firePageChange(prev,curr);	
//		previousPage=newPageIndex;
//	}
	
	/**
	 * Responses to a page change and does the synchronization between 
	 * exploration and trace. That is, the curernt trace state will be 
	 * saved in the old exploration, and new exploration will loaded.
	 * MultiTraceEditor is also refreshed.
	 * @param oldPage - old editor page or null if not exist yet.
	 * @param newPage - new editor page
	 */
	public void firePageChange(TraceEditorPage oldPage,TraceEditorPage newPage) {
		firePageChange(oldPage,newPage,modelView,patternView,utilView);
		refresh();
	}
	
	/**
	 * Synchronize trace and exploration. If views are valid, they will be
	 * refreshed as well.
	 * @param oldPage - old editor page or null if not exist yet.
	 * @param newPage - new editor page
	 * @param mv	- ModelView or null
	 * @param pv	- PatternView or null
	 * @param uv	- UtilityView or null
	 */
	public static void firePageChange(TraceEditorPage oldPage,TraceEditorPage newPage,
					ModelView mv,MethodView pv,MethodView uv) {
		// save old
		if (oldPage!=null) {
			Exploration oldExp=oldPage.getExploration();
			oldExp.setModelView(mv);		// prepare to sync these views
			oldExp.setPatternView(pv);
			oldExp.setUtilityView(uv);
			oldExp.syncExplorationFromTrace();
		}				
		// load new
		if (newPage!=null) {
			if (mv!=null)
				mv.connectEditor(newPage.getMultiPageEditor());
			if (pv!=null)
				pv.connectEditor(newPage.getMultiPageEditor());
			if (uv!=null)
				uv.connectEditor(newPage.getMultiPageEditor());
			Exploration newExp=newPage.getExploration();
			newExp.setModelView(mv);		// prepare to sync these views
			newExp.setPatternView(pv);
			newExp.setUtilityView(uv);
			newExp.syncTraceFromExploration();
			newPage.refresh(); // refresh new the page, old page refresh is ignored
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		getEditor(0).doSave(monitor);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		// TODO Auto-generated method stub
		IEditorPart editor = getEditor(0);
		editor.doSaveAs();
		setPageText(0, editor.getTitle());
		setInput(editor.getEditorInput());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorPart#gotoMarker(org.eclipse.core.resources.IMarker)
	 */
	public void gotoMarker(IMarker marker) {
		// TODO Auto-generated method stub
		getActiveEditor().gotoMarker(marker);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
		throws PartInitException {
		// TODO Auto-generated method stub
		super.init(site, input);
		setInput(input);
		setSite(site);
		traceFile=((FileEditorInput)input).getFile();
		try { 
			traceStream=traceFile.getContents();
		} catch (CoreException ce) {
			SeatUtil.showMessage("Cannot open the resource!");
		}
		setTitleImage(PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT));
	}

	// Trace is loaded and parser of source code is invoked in another thread	
	private Trace loadTrace() {
		Trace trace=loadMethodInvocationTrace(traceStream,traceFile.getName());
		trace.getState().setType(SeatPlugin.getDefault().getExtra().traceType);
		GlobalUtilityFilter guf=new GlobalUtilityFilter();
		guf.apply(trace);

		// Active project must be set before parsing
		activeProject=traceFile.getProject();
		Parser parser=new JavaClassParser(trace,traceFile.getProject(),SeatPlugin.getDefault().getExtra().traceSourceProjects);  //parsing java .class
		IViewPart model=SeatUtil.switchToView(SeatUtil.VIEW_MODEL);
		if (model instanceof ModelView) 
		parser.addParseListener((ModelView)model);	
		parser.parse();
		JavaParser jp=new JavaParser(trace,traceFile.getProject(),SeatPlugin.getDefault().getExtra().traceSourceProjects);  //parsing java source
		jp.parseClassSource(); 

		// make sure initial stateis recoredee internally
		trace.getState().getInitialNodeStates();		
		return trace;
	}
	private Trace loadMethodInvocationTrace(String fileName) {
		long startTime=0,endTime=0;
		startTime = System.currentTimeMillis();
			
		SoftwareSystem system=new SoftwareSystem("IDE","test ide for dev");
		Scenario sce=new Scenario(system,"Update Database"); 
		MethodInvocationTrace trace=new MethodInvocationTrace(sce,new StreamSource(fileName));
		try {
			trace.open();				
		} catch (IOException ioe) {}

		endTime = System.currentTimeMillis();
		System.out.println("Trace(graph) load time: " + (endTime - startTime)/1000.0 +"sec");

		return trace;	
	}	
	private Trace loadMethodInvocationTrace(InputStream ins,String name) {
		try {
		long startTime=0,endTime=0;
		startTime = System.currentTimeMillis();
			
		SoftwareSystem system=new SoftwareSystem("IDE","test ide for dev");
		Scenario sce=new Scenario(system,"Update Database"); 
		MethodInvocationTrace trace=new MethodInvocationTrace(sce,new StreamSource(ins,name,ITraceSource.PERSISTENT));
		trace.open();				

		endTime = System.currentTimeMillis();
		System.out.println("Trace(graph) load time: " + (endTime - startTime)/1000.0 +"sec");
		return trace;	
		} catch (IOException ioe) {
			System.out.println("Failed to open trace");
			return null;
		}
	}			


	/**
	 * Refreshes editor page that is working on. Other pages will be refreshed
	 * as they are actived
	 */
	public void refresh() {
		getCurrentEditor().refresh();
		SeatUtil.refreshStatisViewIfOpened();
		SeatUtil.refreshPropertyViewIfOpened();	
	}
	
	/**
	 * Returns the trace of this editor.
	 */
	public Trace getTrace() {
		// TODO Auto-generated method stub
		return trace;
	}

	/**
	 * Gets the helper model view.
	 */
	public ModelView getModelView() {
		return modelView;
	}

	/**
	 * Sets the helper model view.
	 */
	public void setModelView(ModelView mv) {
		modelView=mv;
	}

	/**
	 * Gets the helper pattern view.
	 */
	public PatternView getPatternView() {
		// TODO Auto-generated method stub
		return patternView;
	}

	/**
	 * Sets the helper pattern view.
	 */
	public void setPatternView(PatternView pv) {
		patternView=pv;
	}

	/**
	 * Gets the helper utility view.
	 */
	public UtilityView getUtilityView() {
		// TODO Auto-generated method stub
		return utilView;
	}

	/**
	 * Sets the helper utility view.
	 */
	public void setUtilityView(UtilityView uv) {
		utilView=uv;
	}

	/**
	 * Gets the active editor page of this multiple page editor.
	 * @return
	 */
	public TraceEditorPage getCurrentEditor() {
		return (TraceEditorPage)getActiveEditor();  
	}
	
	/**
	 * Gets all the editor pages for this trace
	 * @return
	 */
	public TraceEditorPage[] getEditors() {
		int count=getPageCount();
		TraceEditorPage[] pages=new TraceEditorPage[count];
		for (int i=0;i<count;i++) 
			pages[i]=(TraceEditorPage)getEditor(i);
		return pages; 
	}
	
	/**
	 * Gets the project name that this trace residue.
	 * @return
	 */
	public IProject getActiveProject() {
		return activeProject; 
	}	
		
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		
		// Clear the view input to null
		if ((modelView!=null)&&(modelView.getEditor()==this))
				modelView.clear();
		if ((patternView!=null)&&(patternView.getEditor()==this))
				patternView.clear();
		if ((utilView!=null)&&(utilView.getEditor()==this))
				utilView.clear();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
//		if ((part instanceof MultiTraceEditor)&& (part!=previousEditor)) {
//			TraceEditorPage oldPage=null;
//			if (previousEditor!=null)
//				oldPage=previousEditor.getCurrentEditor();
//			TraceEditorPage newPage=((MultiTraceEditor)part).getCurrentEditor();			
//			MultiTraceEditor.firePageChange(oldPage,newPage,modelView,patternView,utilView);
//			((MultiTraceEditor)part).refresh();
//			previousEditor=(MultiTraceEditor)part;			
//		}
	}

	public void partDeactivated(IWorkbenchPart part) {
//		IEditorPart editor=SeatUtil.getActiveEditor();
//		if (!(editor instanceof MultiTraceEditor)) {
//			// remove old list if there
//			Control viewerControl=viewer.getControl();
//			if (viewerControl!=null && !viewerControl.isDisposed()) {
//				viewerControl.setRedraw(false);
//				if (list!=null) {
//					list.clear();
//					viewer.refresh();
//				}
//				viewerControl.setRedraw(true);
//			}
//		}
	}
	
	public void partClosed(IWorkbenchPart part) {partDeactivated(part); }
	public void partOpened(IWorkbenchPart part) {}
	public void partBroughtToTop(IWorkbenchPart part) {	partActivated(part); }

}

