package seat.ui.tree.viewer;

import java.util.ArrayList;
import java.util.Map;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.PropertyDialogAction;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.properties.PropertySheet;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.model.core.dynamic.Trace;
import seat.ui.actions.BookMarkAction;
import seat.ui.actions.CompressAction;
import seat.ui.actions.ExplorationAction;
import seat.ui.actions.HideAction;
import seat.ui.actions.MarkUtilityAction;
import seat.ui.actions.MultiActionContributor;
import seat.ui.algorithm.AlgorithmDescription;
import seat.ui.algorithm.NodeBits;
import seat.ui.preference.UtilityPreference;
import seat.ui.tree.partial.PartialTree;
import seat.ui.tree.partial.TreeNode;
import seat.ui.tree.partial.UniquePath;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class TraceEditorPage extends EditorPart {
	private MultiTraceEditor parentContainer; 
	private TraceViewer traceViewer;
	private boolean defaultEditor; // flag for a menu item, allow delete or not
	private Exploration exploration;

	// context menu actions for 
	private CompressAction[] algActions;			// process subtree
	private MarkUtilityAction utilityAction; 		// mark selected as utility
	private HideAction hideAction; 					// hide selected	
	private BookMarkAction bookmarkAction; 			// book mark selected
	private Action newExploration,delExploration; 	// new exploration or delete exploration 
	private Action refreshAction;					// a force refersh action
	private PropertyDialogAction propertyAction;	// property page	


	private IPropertyChangeListener seatPrefListener =
		new IPropertyChangeListener() {
			/** Implementation of IPropertyChangeListener interface. So the editor will
			 * listen to the property changes. 
			 * @param event
			 */
			public void propertyChange(PropertyChangeEvent event) {
				// TODO Auto-generated method stub
				String property=event.getProperty();
				if (property.equals(SeatUtil.TRACE_SHOWFILTERED)) { // gobal filter on/off
					String filter=SeatUtil.getStringSetting(SeatUtil.TRACE_SHOWFILTERED);
					boolean includeFiltered=Boolean.valueOf(filter).booleanValue();							
					((PartialTree)traceViewer.getTree()).setIncludeHidden(includeFiltered);
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.TRACE_SHORTPROPERTY)) { // short(node only) or long(trace)
					PropertySheet prop=(PropertySheet)SeatUtil.findView(SeatUtil.VIEW_PROPERTY);
					if (prop!=null)
						prop.selectionChanged(parentContainer,traceViewer.getSelection());
				} else if (property.equals(SeatUtil.COLOR_FILTER)) { // colors
					Color changed=SeatUtil.getColorSetting(SeatUtil.COLOR_FILTER);
					SeatPlugin.getDefault().getColorRegistry().put(SeatUtil.COLOR_FILTER,changed);
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.COLOR_HIGHTLIGHT)) {
					Color changed=SeatUtil.getColorSetting(SeatUtil.COLOR_HIGHTLIGHT);
					SeatPlugin.getDefault().getColorRegistry().put(SeatUtil.COLOR_HIGHTLIGHT,changed);
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.COLOR_PATTERN)) {
					Color changed=SeatUtil.getColorSetting(SeatUtil.COLOR_PATTERN);
					SeatPlugin.getDefault().getColorRegistry().put(SeatUtil.COLOR_PATTERN,changed);
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.COLOR_UTILITY)) {
					Color changed=SeatUtil.getColorSetting(SeatUtil.COLOR_UTILITY);
					SeatPlugin.getDefault().getColorRegistry().put(SeatUtil.COLOR_UTILITY,changed);
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.TRACE_ENABLED_DECORATORS)) { // decor, label provider always
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.TRACE_LONG_DECORATION)) {
					traceViewer.refresh(); 					 			
				} else if (property.equals(SeatUtil.TRACE_DECORATION)) {
					traceViewer.refresh(); 					 
				} else if (property.equals(SeatUtil.TRACE_HIDDEN_FLAG)) { 
					traceViewer.refresh(); 		
				} else if (property.equals(SeatUtil.TRACE_UTILITY_METHODLIST)) { // global utility
					UtilityPreference.filterUserUtility(traceViewer.trace);
					traceViewer.refresh();			
				} else { // process for algorithm parameter change			
					processAlgorithmPropertyChange(property);			
				}
			}
	};

	
	/**
	 * Initializes the editor apge with its parent holder and expolarion data
	 */
	public TraceEditorPage(MultiTraceEditor te,Exploration exp) {
		super();
		// TODO Auto-generated constructor stub
		parentContainer=te;
		this.exploration=exp;
		defaultEditor=false;
	}

	
	public TraceEditorPage(MultiTraceEditor te,Exploration exp,boolean def) {
		super();
		// TODO Auto-generated constructor stub
		parentContainer=te;
		this.exploration=exp;
		defaultEditor=def;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		exploration.getSession().save();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		// TODO Auto-generated method stub
		exploration.getSession().save();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorPart#gotoMarker(org.eclipse.core.resources.IMarker)
	 */
	public void gotoMarker(IMarker marker) {
		// TODO Auto-generated method stub
		try {
			Map attrib=marker.getAttributes();
			String edgeIdx=(String)attrib.get(SeatUtil.MARKER_PATH);
			UniquePath path=UniquePath.construct((PartialTree)traceViewer.tree,edgeIdx);
			path.setTrace(exploration.getTrace());
			traceViewer.setSelection(new StructuredSelection(path));
			refresh(path);
		} catch (Exception ce) {
			System.out.println("Can not find path attribute from marker");
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
		throws PartInitException {
		// TODO Auto-generated method stub
		setInput(input);
		setSite(site);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		SeatPlugin.getDefault().getPreferenceStore().removePropertyChangeListener(seatPrefListener);
		PlatformUI.getWorkbench().getPreferenceStore().removePropertyChangeListener(seatPrefListener);
		super.dispose();		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	public boolean isDirty() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		// TODO Auto-generated method stub
		traceViewer=new TraceViewer(parent,exploration);
		SeatPlugin.getDefault().getPreferenceStore().addPropertyChangeListener(seatPrefListener);
		PlatformUI.getWorkbench().getPreferenceStore().addPropertyChangeListener(seatPrefListener);
		
		// for property page to work
		getSite().setSelectionProvider(traceViewer);
		// add right click support
		hookContextMenu();
		traceViewer.refresh();
		}
		


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// TODO Auto-generated method stub
		traceViewer.getControl().setFocus();

	}
	
	/**
	 * Gets the exploratino associates with this page
	 * @return
	 */	
	public Exploration getExploration() {
		return exploration;
	}
	
	/**
	 * Returns the trace that this page is editing.
	 * @return
	 */
	public Trace getTrace() {
		return traceViewer.getTrace();
	}
	
	/**
	 * Returns parent container this page is inside.
	 * @return
	 */
	public MultiTraceEditor getMultiPageEditor() {
		return parentContainer;
	}

	/**
	 * Returns the main viewer of this editor page
	 * @return
	 */
	public TraceViewer getViewer() {
		return traceViewer;
	}
	
	/**
	 * Refreshes editor to reflect data changes. Internally it delegates
	 * calls to the viewer to do the work.
	 */
	public void refresh() {
		traceViewer.refresh();
	}

	/**
	 * Refreshed the editor with the data from given path as the first 
	 * node in the editor page. Internally it delegates calls to viewer
	 * and gets model refreshed and rendered.
	 * @param top
	 */
	public void refresh(UniquePath top) {
		traceViewer.refresh(top);
		}
		
	/*
	 * Creates all the actions for the context menu.
	 */	
	private void createActions() {
		AlgorithmDescription[] ads=SeatPlugin.getAlgorithmDescriptions();
		ArrayList ca=new ArrayList();
		for (int i=0;i<ads.length;i++) {
			if (ads[i].getCompressSubtree()!=null)
				ca.add(new CompressAction(this,ads[i])); 
		}			
		algActions=(CompressAction[])ca.toArray(new CompressAction[ca.size()]);
		
		utilityAction=new MarkUtilityAction(this); 
		hideAction=new HideAction(this);
		newExploration=new ExplorationAction(this,1);
		delExploration=new ExplorationAction(this,0);		
		bookmarkAction=new BookMarkAction(this);		

		refreshAction=new Action() {
			public void run() {
				((PartialTree)traceViewer.getTree()).refreshAllExpansion(true);
				traceViewer.refresh();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refresh All Expanded Nodes");
		refreshAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/refresh.gif"));
		
		propertyAction = new PropertyDialogAction(getSite().getShell(),traceViewer);
		propertyAction.setText("Properties");
		propertyAction.setToolTipText("Show detailed propertiess");
		propertyAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
					getImageDescriptor(ISharedImages.IMG_OBJS_TASK_TSK));
	}	

	/*
	 * Hooks context menu actions  
	 */
	private void hookContextMenu() {
		createActions();
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				IStructuredSelection selection=(IStructuredSelection)traceViewer.getSelection();
				if (selection==null)
					return;
				TreeNode current=(TreeNode)selection.getFirstElement();
				NodeBits nb=(NodeBits)exploration.getCachedNodeContext(current);
				for (int i=0;i<algActions.length;i++) {
					algActions[i].setChecked(nb.isApplied(algActions[i].getAlgorithmIndex()));
					manager.add(algActions[i]);											
				}
				
				manager.add(new Separator());
				utilityAction.setChecked(nb.isApplied(SeatUtil.RESERVED_ALGIDX_UTILITY)
				&&current.getElement().isHiddenBy(SeatUtil.RESERVED_ALGIDX_UTILITY));
				manager.add(utilityAction);
				hideAction.setChecked(nb.isApplied(SeatUtil.RESERVED_ALGIDX_FILTER)
				||current.getElement().isHidden());//isHiddenBy(SeatUtil.RESERVED_ALGIDX_FILTER));
				manager.add(hideAction);
				manager.add(bookmarkAction);

				manager.add(new Separator());
				manager.add(refreshAction);				
				manager.add(newExploration);
				if (!defaultEditor)
					manager.add(delExploration);

				manager.add(new Separator());
				manager.add(propertyAction);				
				manager.add(new Separator("Additions"));						
			}
		});
		Menu menu = menuMgr.createContextMenu(traceViewer.getControl());
		traceViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, traceViewer);
	}	
	
	/** 
	 * Identify the algorithm this property belongs to by extract index from property.
	 * index@#@property. for all opened ITraceEditors, recall apply or undo with new parameter
	 */
	private void processAlgorithmPropertyChange(String property) {
		String index,prop;
		int sig=property.indexOf("@#@");
		if (sig==-1) // not a parameter 
			return;
			
		// reapply all algoirthms that are on because they may interfere with each other
		MultiActionContributor.reapply4CurrentEditor();
//		index=property.substring(0,sig);
//		prop=property.substring(sig+3); // actuall not used, load from store
//
//		AlgorithmDescription algorithmDesc=AlgorithmList.getAlgorithmDescription(index);		
//		Parameter[] paras=algorithmDesc.getParameters();
//		for (int i=1;i<paras.length;i++) { // o is for internal index
//			paras[i].setCurrent(SeatUtil.getStringSetting(index+"@#@"+paras[i].getID()));
//		}			
//		Algorithm algorithm=algorithmDesc.createAlgorithm();		
//		algorithm.init(paras);		
//
//		IEditorReference[] refer=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
//		Trace contained=null;
//		for (int i=0;i<refer.length;i++) {
//			IEditorPart ep=refer[i].getEditor(true);
//			if (ep instanceof ITraceEditor) {
//				ITraceViewer viewer=((ITraceEditor)ep).getViewer();
//				boolean algorthimApplied=viewer.getTrace().getState().getAlgorithmList().getAlgorithmStatus(index);
//				if (algorthimApplied) {
//					algorithm.undo(viewer.getTrace());
//					algorithm.apply(viewer.getTrace());
//				}
//			}
//		}
//		pviewer.refreshViewerWithReload();
	}
	
//	public void reapplyAllAlgorithms() {
////		IEditorReference[] refer=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
////		Trace contained=null;
////		for (int i=0;i<refer.length;i++) {
////			IEditorPart ep=refer[i].getEditor(true);
////			if (ep instanceof ITraceEditor) {
////				ITraceViewer viewer=((ITraceEditor)ep).getViewer();
////				AlgorithmList al=viewer.getTrace().getState().getAlgorithmList();
////				for (int kk=0;kk<al.getNumOfAlgorithmDescription();kk++) {
////					boolean algorthimApplied=al.getAlgorithmStatus(kk);
////					if (algorthimApplied) {
////						algorithm.undo(viewer.getTrace());
////						algorithm.apply(viewer.getTrace());
////					}
////				}
////			}
////		}
//		
//	}

}
