/*
 * Created on Sep 15, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package seat.ui.tree.widget;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import seat.ui.tree.partial.PartialTree;
import seat.ui.tree.partial.TreeNode;
import seat.ui.tree.viewer.IExpandProvider;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PartialTreeViewer extends ContentViewer {
	
	private PictureTree pictureTree;
	
	private ArrayList doubleClickListeners = new ArrayList(1);


	/**
	 * 
	 */
	public PartialTreeViewer(Composite parent) {
		this(new PictureTree(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL));
	}
	public PartialTreeViewer(PictureTree tree) {
		super();
		this.pictureTree = tree;
		this.pictureTree.setViewer(this);
	}

	/**
	 * Adds a listener for double-clicks in this viewer.
	 * Has no effect if an identical listener is already registered.
	 *
	 * @param listener a double-click listener
	 */

	public void addDoubleClickListener(IDoubleClickListener listener) {
		if (!doubleClickListeners.contains(listener)) 
			doubleClickListeners.add(listener);
	}
	
	/**
	 * Removes the given double-click listener from this viewer.
	 * Has no affect if an identical listener is not registered.
	 *
	 * @param listener a double-click listener
	 */
	public void removeDoubleClickListener(IDoubleClickListener listener) {
		doubleClickListeners.remove(listener);
	}
	
	/**
	 * Notifies any double-click listeners that a double-click has been received.
	 * Only listeners registered at the time this method is called are notified.
	 *
	 * @param event a double-click event
	 *
	 * @see IDoubleClickListener#doubleClick
	 */
	public void fireDoubleClick(final DoubleClickEvent event) {
		Object[] listeners = doubleClickListeners.toArray();
		for (int i = 0; i < listeners.length; ++i) {
			final IDoubleClickListener l = (IDoubleClickListener)listeners[i];
			Platform.run(new SafeRunnable() {
				public void run() {
					l.doubleClick(event);
				}
				public void handleException(Throwable e) {
					super.handleException(e);
					//If and unexpected exception happens, remove it
					//to make sure the workbench keeps running.
					removeDoubleClickListener(l);
				}
			});			
		}
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getControl()
	 */
	public Control getControl() {
		// TODO Auto-generated method stub
		return pictureTree;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	public ISelection getSelection() {
		// TODO Auto-generated method stub
		PictureLabel[] labels=pictureTree.getSelection();
		int len=(labels==null)?0:labels.length;
		Object[] objs=new Object[len];
		if (labels!=null)
			for (int i=0;i<labels.length;i++)
				objs[i]=labels[i].getData();
		return new StructuredSelection(objs);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	public void refresh() {
		// TODO Auto-generated method stub
		// prepare loading new pictureTree
		pictureTree.refreshPartialTree(0);
		determineAppearance();
		pictureTree.setRedraw(true);
		pictureTree.redraw();
	} 

	public void refresh(TreeNode top) {
		// TODO Auto-generated method stub
		// prepare loading new pictureTree
		pictureTree.refreshPartialTree(top);
		determineAppearance();
		pictureTree.setRedraw(true);
		pictureTree.redraw();
	} 
	
	/**
	 * Calls label provider to get image/color/label for a node.
	 * It calls a setImages()(default is empty) to prepare any images setttings. 
	 * SubClass can reimplementing setImages(). 
	 */
	public void determineAppearance() {
		setImages();
		IBaseLabelProvider base=getLabelProvider();
		if (base instanceof ILabelProvider) {
			ILabelProvider labelProv=(ILabelProvider)base;
			for (int i=0;i<pictureTree.picLabelList.size();i++) {
				PictureLabel pl=(PictureLabel)pictureTree.picLabelList.get(i);
				Object element=pl.getData();
				pl.setImage(labelProv.getImage(element));
				pl.setText(labelProv.getText(element));
				if (labelProv instanceof IColorProvider) {
					IColorProvider cp = (IColorProvider) labelProv;
					pl.setForeground(cp.getForeground(element));
					pl.setBackground(cp.getBackground(element));				
				}
				if (labelProv instanceof IExpandProvider) {
					IExpandProvider ep = (IExpandProvider) labelProv;
					pl.setExpandIcon(ep.getExpandID(element));
				}				
			}
		}
	}
	protected void setImages() {
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers.ISelection, boolean)
	 */
	public void setSelection(ISelection selection, boolean reveal) {
		// TODO Auto-generated method stub
		if (!selection.isEmpty()&&(selection instanceof StructuredSelection)) {
			StructuredSelection ss=(StructuredSelection)selection;
			ArrayList pls=new ArrayList();
			Iterator it=ss.iterator();
			while (it.hasNext()) {
				Object obj=it.next();
				if (obj instanceof PictureLabel) 
					pls.add(obj);
			}
			pictureTree.setSelection((PictureLabel[])pls.toArray(new PictureLabel[pls.size()]));
		}
	}

	public void setSelection(ISelection selection) {
		setSelection(selection,true);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setInput(java.lang.Object)
	 */
	public void setInput(Object input) {
		// TODO Auto-generated method stub
		super.setInput(input);
		if (input instanceof PartialTree) {
			pictureTree.setPartialTree((PartialTree)input);
			refresh();
		}
			
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#fireSelectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void fireSelectionChanged(SelectionChangedEvent event) {
		// TODO Auto-generated method stub
		long startTime=0,endTime=0;
		startTime = System.currentTimeMillis();
		
		super.fireSelectionChanged(event);
		endTime = System.currentTimeMillis();
//		System.out.println("Selection Changed time: " + (endTime - startTime)/1000.0 +"sec");
		startTime=endTime;
		
	}

}
