package seat.ui.tree.widget;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import seat.SeatUtil;
import seat.ui.tree.partial.TreeNode;

// level, row starts from 1
public class PictureLabel {
	private static Image[] EXPANSION= { SeatUtil.getImageDescriptor("icons/noexpand.gif").createImage(), 
										SeatUtil.getImageDescriptor("icons/plus.gif").createImage(),
										SeatUtil.getImageDescriptor("icons/minus.gif").createImage(),
										SeatUtil.getImageDescriptor("icons/wave.gif").createImage() }; 
	public static final int NONE=0;       // expand icons
	public static final int PLUS=1; 
	public static final int MINUS=2; 
	public static final int WAVE=3; 
	public static final int SPACING=3;
	public static final int ICONSIZE=16;
	private int expandId;		    // expand icons id:0-3 
	private Image nodeImage;		// iamge for this node
	private String label;  			// lable text
	private int level=0;			// level of this node, starts from 0, for computing indent in x 
	private int row=0;				// row number, starts from 0, for computing y
	private Color background;
	private Color foreground;
	private TreeNode treePath;	// model data associated with this node
	
	private Composite container;	// parent, used to determine the width of this node, height is always ICONSIZE

	public PictureLabel(Composite parent,int exp, Image node, String label, int level, int row) {
		container=parent;
		if (exp>3)
			expandId=0;
		else 
			expandId=exp;
		this.nodeImage=node;
		this.label=label;		
		this.level=level;
		this.row=row;
		this.background=Display.getCurrent().getSystemColor(SWT.COLOR_WHITE);
		this.foreground=Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
	}
	

	/**
	 * @return
	 */
	public Color getBackground() {
		return background;
	}

	public TreeNode getData() {
		return treePath;
	}

	/**
	 * @return
	 */
	public int getExpandId() {
		return expandId;
	}
	/**
	 * @return
	 */
	public boolean getExpanded() {
		return (expandId==MINUS)||(expandId==WAVE)?true:false;
	}	
	public Image getExpandImage() {
		return EXPANSION[expandId];
	}

	/**
	 * @return
	 */
	public Color getForeground() {
		return foreground;
	}

	public Image getImage() {
		return nodeImage;
	}
	/**
	 * @return
	 */
	public String getText() {
		return label;
	}
	/**
	 * @return
	 */
	public int getLevel() {
		return level;
	}

	/**
	 * Spacing in X 
	 * @return
	 */
	public int getIndent() {
		return level*ICONSIZE;
	}
	/*
	 * Bounds starting from expand icon till end of label, without indent
	 */
	public Rectangle getBounds() {
		int x=getIndent();
		int y=row*ICONSIZE;
		GC gc=new GC(container);
		int width =ICONSIZE*2+gc.textExtent(label).x;
		gc.dispose(); 
		return new Rectangle(x,y,width,ICONSIZE);
	}
	/*
	 * Bounds from expand icon only
	 */
	public Rectangle getExpandBounds() {
		int x=getIndent();
		int y=row*ICONSIZE;
		return new Rectangle(x,y,ICONSIZE,ICONSIZE);
	}	
	
	/*
	 * Bounds for image
	 */
	public Rectangle getImageBounds() {
		int x=getIndent()+ICONSIZE;
		int y=row*ICONSIZE;
		return new Rectangle(x,y,ICONSIZE,ICONSIZE);
	}		
	/*
	 * Bounds for texxt label
	 */
	public Rectangle getTextBounds() {
		int x=getIndent()+ICONSIZE*2;
		int y=row*ICONSIZE;
		GC gc=new GC(container);
		int width=gc.textExtent(label).x;
		gc.dispose(); 
		return new Rectangle(x,y,width,ICONSIZE);
	}	
	
	/**
	 * @return
	 */
	public int getRow() {
		return row;
	}

	
	public void setData(TreeNode element) {
		treePath=element;
	}

	public void setImage(Image img) {
		nodeImage=img;
	}
	public void setText(String s) {
		label=s;
	}
	
	public void setExpandIcon(int id) {
		if (id>3)
			expandId=0;
		else 
			expandId=id;
	}
	
	public void setLevel(int lev) {
		level=lev;
	}
	public void setForeground(Color fore) {
		foreground=fore;
	}
	public void setBackground(Color back) {
		background=back;
	}
	
	public void restoreDefaultColor() {
		this.background=Display.getCurrent().getSystemColor(SWT.COLOR_WHITE);
		this.foreground=Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);		
	}
	
	/**
	 * @param i
	 */
	public void setRow(int i) {
		row = i;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object arg0) {
		// TODO Auto-generated method stub
		if (arg0==null) {
			return (treePath==null)? true:false;
		} else if ((treePath==null)||!(arg0 instanceof PictureLabel))
			return false;
		else 
			return treePath.equals(((PictureLabel)arg0).treePath);
	}

}
