package seat.ui.views;


import java.util.ArrayList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.Assert;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import seat.SeatPlugin;
import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.model.core.structure.ClassElement;
import seat.model.core.structure.LogicalPackage;
import seat.model.core.structure.MethodElement;
import seat.model.core.structure.StructuralElement;
import seat.ui.actions.GroupHideAction;
import seat.ui.algorithm.JavaParser;
import seat.ui.algorithm.ParseListener;
import seat.ui.algorithm.SourceParseEvent;
import seat.ui.tree.partial.PartialTree;
import seat.ui.tree.partial.UniquePath;
import seat.ui.tree.viewer.MultiTraceEditor;

/**
 * This view displays model data, including nodes, classes and packages using a CheckboxTableViewer.
 * This view class implements ViewContentProvider, ViewLableProvider, ViewSelectionChangeListner
 * and NodeSorter as its helper classes.
 * @author lfu
 * @see org.kbre.seat.views.TreeView
 */
public class ModelView extends ViewPart implements IPartListener, ParseListener{
	protected CheckboxTableViewer viewer;				// main viewer
	protected ArrayList list;         	// input of viewer	
	protected MultiTraceEditor editor,previous=null; 	// Editor that associated with this view
	
	private String xmlTitle;							// default title string
	private PathGroup pathGroup;					// PathGroup of current selected node 
	private Node highlighted;						// current selected node

	// all the actions will be created in makeActions();
	private Action nextAction;							
	private Action prevAction;
	private Action doubleClickAction;
	private Action hideAction;
	private Action showAction;
	private Action filterAction;
	private Action selectAllAction;
	private Action findFirstAction;
	private Action refreshAction;
	private NodeSorter occurSorter;  // specifically extra for sort action
	private ModelFilter modelFilter;
	
	private ArrayList filteredElements=new ArrayList();
	private ArrayList lightBulb=new ArrayList();
	private ArrayList darkBulb=new ArrayList();

	// Viewerer selection changed listener, see createPartControl()
	private ISelectionChangedListener viewSelectionChangedListener 
		=new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			// TODO Auto-generated method stub
			IStructuredSelection selection=(IStructuredSelection)event.getSelection();
			Object first=selection.getFirstElement();
			if ( (first instanceof Node) &&(editor!=null)){
				Node nn=(Node)first;
				MethodElement me=(MethodElement)nn.getBehaviouralElement();
				viewer.getTable().setToolTipText(me.getDescription());				
				setHighLightedNode((Node)first);
				doubleClickAction.run(); // perform a first search
				updateActionStatus();
//				System.out.println("Total="+temp);
			}
		}
	};
	
	// Listerns model hide/show command
	private ICheckStateListener checkStateListener =new ICheckStateListener() {
		public void checkStateChanged(CheckStateChangedEvent event) {
			Object model=event.getElement();
			boolean hide=event.getChecked();
			usrSelectedShowHide(model,!hide);
			viewer.setSelection(new StructuredSelection(model),true);
			updateTitle();
			viewer.refresh();		
			
			if (editor!=null) 
				editor.refresh();
		}
	};


	/**
	 * When initialized, this view adds itself as listners to parse event.
	 */
	public void sourceParsed(SourceParseEvent event) {
			viewer.refresh();
		}
	/**
	 * Provides contents for the viewer 
	 */
	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		/**
		 *  Input is from ArrayList, returned as an array.
		 */
		public Object[] getElements(Object parent) {
			return (list==null)?new Object[0]:list.toArray();
		}
	}
	
	/**
	 * Provides labels and images for EUtilityOccurrence objects in view.
	 * @see org.eclipse.jface.viewers.LabelProvider
	 */
	int temp=0;
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider,IColorProvider {
		private Image imgPackage=SeatPlugin.getImage(SeatUtil.ICON_PACKAGE);
		private Image imgClass=SeatPlugin.getImage(SeatUtil.ICON_CLASS);
		private Image imgCompound=SeatPlugin.getImage(SeatUtil.ICON_COMPOUND);
		private Image imgMethod=SeatPlugin.getImage(SeatUtil.ICON_METHOD);
		private Color white=Display.getCurrent().getSystemColor(SWT.COLOR_WHITE);
		private Color black=Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
		private Color gray=Display.getCurrent().getSystemColor(SWT.COLOR_GRAY);

		/** 
		 * Returns the source name of Exploration as labels
		 * See Exploration 
		 */
		public String getColumnText(Object obj, int index) {
			if (obj instanceof Node) {
					Node node=(Node)obj;
					switch (index) {
						case 0:
							return "";
						case 1:
							return node.getLabel();
						case 2:
							int par=PathGroup.count(node);//node.getNumberOfParents();
							int occ=(par>0)?par:1;
							temp+=occ;
							return (par>0)?par+"":"1";
						case 3:
//							MethodElement me=(MethodElement)node.getBehaviouralElement();
//							return me.getDescription();	
							String comment=JavaParser.getMethodComment(editor.getTrace(),node);
							return (comment==null)?"":comment;
						default:
							return "";
					}
			} else if (obj instanceof ClassElement) {
				ClassElement ce=(ClassElement)obj;
						switch (index) {
							case 0:
								return "";
							case 1:
								return ce.getName();
							case 2:
								return "Class";
							case 3:
								String comment=JavaParser.getClassComment(editor.getTrace(),ce.getName());
								return (comment==null)?"":comment;
							default:
								return "";
						}				
			} else if (obj instanceof LogicalPackage) {
				LogicalPackage lp=(LogicalPackage)obj;
					switch (index) {
						case 0:
							return "";
						case 1:
							return lp.getName();
						case 2:
							return "Package";
						case 3:
							return "";
						default:
							return "";
					}				
			}
			return obj.toString();								
		}

		/** 
		 * Returns a image for a given column for the exploration view. The primary viewer in
		 * exploration view is a table viewer.   
		 */
		public Image getColumnImage(Object obj, int index) {
			if (index!=1)
				return null;
			if (obj instanceof Node) {
				Node node=(Node)obj;				
				return (node.getNumberOfChildren()>0)?imgCompound:imgMethod;
			} else if (obj instanceof ClassElement) {
				return imgClass;
			} else 
				return imgPackage;
		}
	
		public Color getForeground(Object element) {
				return black; 			
		}
		public Color getBackground(Object element) {
			return (filteredElements.contains(element))?gray:white;
		}
	}

	class NodeSorter extends ViewerSorter {
		private final static int CHECK			= 0;
		private final static int NAME			= 1;
		private final static int OCCURRENCE		= 2;
		private final static int DESCRIPTION 	= 3;
		private final static int CHECK2			= 10;
		private final static int NAME2			= 11;
		private final static int OCCURRENCE2	= 12;
		private final static int DESCRIPTION2 	= 13;

		// Criteria that used to identify a comaprison
		private int criteria;

		/*
		 * Constructor argument values that indicate to sort items by 
		 * description, owner or percent complete.
		 */
		public NodeSorter(int criteria) {
			super();
			this.criteria = criteria;
		}

		/* (non-Javadoc)
		 * Method declared on ViewerSorter.
		 */
		public int compare(Viewer viewer, Object po1, Object po2) {
			switch (criteria) {
				case CHECK:
					return compareCheck(po1, po2);
				case NAME:
					return compareName(po1, po2);
				case OCCURRENCE:
					return compareOccurrence(po1, po2);
				case DESCRIPTION :
					return compareDescription(po1, po2);
				case CHECK2:
					return compareCheck(po2, po1);
				case NAME2:
					return compareName(po2, po1);
				case OCCURRENCE2:
					return compareOccurrence(po2, po1);
				case DESCRIPTION2 :
					return compareDescription(po2, po1);
				default:
					return 0;
			}
		}

		private int compareCheck(Object po1, Object po2) {
			int s1=(viewer.getChecked(po1))?1:0;
			int s2=(viewer.getChecked(po2))?1:0;
			
			return s1-s2;
		}

		private int compareName(Object po1, Object po2) {
			String s1="",s2="";
			if (po1 instanceof Node)
				s1=((Node)po1).getLabel();
			else if (po1 instanceof StructuralElement) 
				s1=((StructuralElement)po1).getName();
			else s1=po1.toString();
			if (po2 instanceof Node)
				s2=((Node)po2).getLabel();
			else if (po2 instanceof StructuralElement) 
				s2=((StructuralElement)po2).getName();
			else s2=po2.toString();
			
			return collator.compare(s1,s2);
		}
		private int compareOccurrence(Object po1, Object po2) {
			int s1=0,s2=0;
			if (po1 instanceof Node)
				s1=((Node)po1).getNumberOfParents();
			else s1=1;
			if (po2 instanceof Node)
				s2=((Node)po2).getNumberOfParents();
			else s2=1;
			return s1-s2;
		}
		private int compareDescription(Object po1, Object po2) {
			return 0;//collator.compare(po1.getDescription(), po2.getDescription());
		}
	}

	class ModelFilter extends ViewerFilter {
		private int filterType=0; // 0-all;1-vis:2-invis
		
		public void setFilterType(int type) {
			filterType=type;		 
		}
		public boolean select(Viewer viewer,Object parentElement,Object element) {
			switch (filterType) {
				case 0:
					return true;
				case 1:
					return !filteredElements.contains(element);
				case 2:
					return filteredElements.contains(element);
				default:
					return true;
			}
		}		
	}	
	
	/**
	 * 
	 */
	public ModelView() {
		super();
		// TODO Auto-generated constructor stub
		IEditorPart edit=SeatUtil.getActiveEditor();
		if ( (edit!=null)&& (edit instanceof MultiTraceEditor) ) {
			connectEditor((MultiTraceEditor)edit);
		}				
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 * 
	 */
	public void createPartControl(Composite parent) {
		int style = SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
					SWT.FULL_SELECTION |SWT.MULTI|SWT.CHECK;		

		Table table=new Table(parent,style);//(Table)viewer.getControl();
		table.setLinesVisible (true);
		table.setHeaderVisible (true);
		String[] titles = {"Hidden", "Name", "Occurrence", "Source Code Comments" };
		
		TableColumn column = new TableColumn (table, SWT.LEFT|SWT.CHECK,0);
		column.setText ("Hidden");
		column.setWidth (45);
		column = new TableColumn (table, SWT.LEFT,1);
		column.setText ("Name");
		column.setWidth (250);
		column = new TableColumn (table, SWT.LEFT,2);
		column.setText ("Occurrence");
		column.setWidth (100);
		column = new TableColumn (table, SWT.CENTER,3);
		column.setText ( "Source Code Comments");
		column.setWidth (300);
		viewer = new CheckboxTableViewer(table); //SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.HIDE_SELECTION);

		viewer.setUseHashlookup(true);		
		viewer.setColumnProperties(titles);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.addSelectionChangedListener(viewSelectionChangedListener);
		viewer.addCheckStateListener(checkStateListener);
		modelFilter=new ModelFilter();
		viewer.addFilter(modelFilter);
		setViewerSorter();
		viewer.setInput(list);		
		// add self as the listener to the editor 
		this.getSite().getPage().addPartListener(this);				

		// actions&menus
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		hookDoubleClickAction();
		xmlTitle=this.getTitle();
		if (list!=null && list.size()>0)
			updateTitle();
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
		super.dispose();
		disconnectEditor();
		this.getSite().getPage().removePartListener(this);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		Control viewerControl=viewer.getControl();
		if (viewerControl!=null && !viewerControl.isDisposed()) 
			viewerControl.setFocus();
	}
	
	private void setFilter(int type) {
		viewer.getControl().setRedraw(false);
		modelFilter.filterType=type;
		viewer.refresh();
		viewer.setCheckedElements(filteredElements.toArray());
		viewer.getControl().setRedraw(true);
	}

	// part listener interface, partActivated() need to be implemented in subclases.
	public void partActivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		if ((part instanceof MultiTraceEditor)&& (part!=previous)) {
			editor=(MultiTraceEditor)part;
			previous=editor;

			// remove old list if there
			Control viewerControl=viewer.getControl();
//			if (viewerControl!=null && !viewerControl.isDisposed()) {
//				viewerControl.setRedraw(false);
//				if (list!=null) {
//					list.clear();
//					viewer.refresh();
//				}
//			}
			
			//add new
			connectEditor(editor);

			if (viewerControl!=null && !viewerControl.isDisposed()) {
					viewerControl.setRedraw(false);
					viewer.setInput(list);
					viewerControl.setRedraw(true);
				}

			editor.refresh();
			updateTitle();						
		}		
	}
	public void partDeactivated(IWorkbenchPart part) {
//		IEditorPart editor=SeatUtil.getActiveEditor();
//		if (!(editor instanceof MultiTraceEditor)) {
//			// remove old list if there
//			Control viewerControl=viewer.getControl();
//			if (viewerControl!=null && !viewerControl.isDisposed()) {
//				viewerControl.setRedraw(false);
//				if (list!=null) {
//					list.clear();
//					viewer.refresh();
//				}
//				viewerControl.setRedraw(true);
//			}
//		}
	}
	
	public void partClosed(IWorkbenchPart part) {partDeactivated(part); }
	public void partOpened(IWorkbenchPart part) {}
	public void partBroughtToTop(IWorkbenchPart part) {	partActivated(part); }

	public void refresh(Object input) {
		Control viewerControl=viewer.getControl();
		if (viewerControl!=null && !viewerControl.isDisposed()) {
			viewerControl.setRedraw(false);
			list=(ArrayList)input;
			viewer.setInput(list);
			viewer.refresh();
			viewerControl.setRedraw(true);
			updateTitle();
		}		 
	}
	
	public void clear() {
		refresh(null);
	}

	public void updateTitle() {
		int occ, invisible;
		occ=(list==null)?0:list.size();
		invisible=viewer.getCheckedElements().length;
		this.setTitle(xmlTitle+" ("+ occ+" occurrences of packages/classes/methods: "+
					  (occ-invisible) + " are shown, " +invisible +" are hidden)");		
		updateActionStatus();
	}

	// Creates actions defined in the class
	protected void makeActions() {
		// TODO Auto-generated method stub
		final ModelView current=this;
		doubleClickAction = new Action() {  // action inherit from parent class
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				highlighted=((Node)obj);
				if (editor!=null) {
					PartialTree ptree=(PartialTree)editor.getCurrentEditor().getViewer().getTree();
					pathGroup=new PathGroup(ptree,highlighted);
					if (pathGroup.hasNext()) {
						UniquePath path=pathGroup.next();
						displayNextTree(path);
					} else if (SeatUtil.getActivePart()==current)
						SeatUtil.showMessage("No more matches found!");
				}
				doubleClicked();
			}
		};			
		// promote to global
		nextAction = new Action() {
			public void run() {
				if (highlighted!=null) {
					if (editor!=null) {
						PartialTree ptree=(PartialTree)editor.getCurrentEditor().getViewer().getTree();
						if (pathGroup==null)
							pathGroup=new PathGroup(ptree,highlighted);
						if (pathGroup.hasNext()) {
							UniquePath path=pathGroup.next();
							displayNextTree(path);
						} else if (SeatUtil.getActivePart()==current)
							SeatUtil.showMessage("No more matches found!");
					}
				}
				nextActionClicked();					
			}
		};
		nextAction.setText("Go to Next Occurrence");
		nextAction.setToolTipText("Go to Next Occurrence");
		nextAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/next.gif"));
		
		prevAction = new Action() {
			public void run() {
				if (highlighted!=null) {
					if (editor!=null) {
						PartialTree ptree=(PartialTree)editor.getCurrentEditor().getViewer().getTree();
						if (pathGroup==null)
							pathGroup=new PathGroup(ptree,highlighted);
						if (pathGroup.hasPrevious()) {
							UniquePath path=pathGroup.previous();
							displayNextTree(path);
						} else if (SeatUtil.getActivePart()==current)
							SeatUtil.showMessage("No more matches found!");
					}
				}
				prevActionClicked();		

			}
		};
		prevAction.setText("Go to Previous Occurrence");
		prevAction.setToolTipText("Go to Previous Occurrence");
		prevAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/prev.gif"));

		// hide all
		hideAction= new Action() {
			public void run() {
				IStructuredSelection selection=(IStructuredSelection)viewer.getSelection();
				Object[] hide=selection.toArray();
				hideUserSelectedNodes(hide);									
			}
		};
		hideAction.setText("Hide");
		hideAction.setToolTipText("Hide Selected Nodes");
		hideAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/fields_co.gif"));

		// show all
		showAction= new Action() {
			public void run() {
				IStructuredSelection selection=(IStructuredSelection)viewer.getSelection();
				Object[] hide=selection.toArray();
				showUserSelectedNodes(hide);
			}
		};
		showAction.setText("Show");
		showAction.setToolTipText("Show Selected Nodes");
		showAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/show.gif"));

		// show all
		filterAction= new Action() {
			public void run() {
				ModelFilterDialog mdf=new ModelFilterDialog(getViewSite().getShell());
				if (mdf.open()==Window.OK) {
					int type=mdf.getValue();
					setFilter(type);
				}					
			}
		};
		filterAction.setText("Filters...");
		filterAction.setToolTipText("Filters...");
		filterAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/filter_tsk.gif"));
		
		selectAllAction= new Action() {
			public void run() {
				viewer.getTable().selectAll();
			}
		};	
		selectAllAction.setText("Select All");
		selectAllAction.setToolTipText("Select All");
		
		findFirstAction= new Action() {
			public void run() {
				String userSelection=GroupHideAction.userSelectionDialog("Find First Matched","Enter the name of to be searched");
				if (userSelection==null) // canceled
					return;
				Table table=viewer.getTable();	
				for (int i=0;i<table.getItemCount();i++) {
					String label=table.getItem(i).getText(1);
					if (label.indexOf(userSelection)!=-1) { 
						table.setSelection(i);
						return;
					}
				}
				SeatUtil.showMessage("No match found!");					
			}
		};
		findFirstAction.setText("Find...");
		findFirstAction.setToolTipText("Find First Matched From Current Position");
		findFirstAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/find1.gif"));

		refreshAction=new Action() {
			public void run() {
				viewer.refresh();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refresh editor");
		refreshAction.setImageDescriptor(SeatUtil.getImageDescriptor("icons/refresh.gif"));		
	}
	
		
	// Adds context menu
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenuExpl");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	// Adds double click funtion to the viewer
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (doubleClickAction!=null)
					doubleClickAction.run();
			}
		});
	}

	// Adds items to the pulldown menu and toolbar of this view
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	// Hooks to the pulldown, called by hookcontextMenu
	protected void fillContextMenu(IMenuManager manager) {
		manager.add(hideAction);
		manager.add(showAction);
		manager.add(selectAllAction);		
		manager.add(refreshAction);		
		manager.add(new Separator("Additions")); // a must for context menu		
	}
	
	// Adds actions to the local pulldown menu
	protected void fillLocalPullDown(IMenuManager manager) {
		// TODO Auto-generated method stub
		// no extra for now
	} 

	// Adds actions to local tool bar 
	protected void fillLocalToolBar(IToolBarManager manager){
		manager.add(nextAction);
		manager.add(prevAction);
		manager.add(hideAction);
		manager.add(showAction);
		manager.add(filterAction);
		manager.add(findFirstAction);
		manager.add(refreshAction);
	}

	// update action enablement for the local toolbar of the view
	private void updateActionStatus(){
		if ((list==null)||(list.size()==0)) { // may have not set yet
			nextAction.setEnabled(false);
			prevAction.setEnabled(false);
			filterAction.setEnabled(false);
			findFirstAction.setEnabled(false);
			return;
		}

//		IStructuredSelection sel=(IStructuredSelection)viewer.getSelection();
//		Node node=(Node)sel.getFirstElement();
//		if (node!=null) {
//			if (node.isHidden()&&!Boolean.valueOf(SeatUtil.getStringSetting(SeatUtil.TRACE_SHOWFILTERED)).booleanValue()) {
//				nextAction.setEnabled(false);
//				prevAction.setEnabled(false);
//			} else {
//			}
//		}
				
		nextAction.setEnabled(true);
		prevAction.setEnabled(true);
		filterAction.setEnabled(true);
		findFirstAction.setEnabled(true);		
	}
	
	// set sorted for the table
	private void setViewerSorter() {
		final NodeSorter checkSorter,nameSorter,descSorter;
		final NodeSorter checkSorter2,nameSorter2, occurSorter2,descSorter2;			
		checkSorter=new NodeSorter(NodeSorter.CHECK);
		nameSorter=new NodeSorter(NodeSorter.NAME);
		occurSorter=new NodeSorter(NodeSorter.OCCURRENCE);
		descSorter=new NodeSorter(NodeSorter.DESCRIPTION);
		checkSorter2=new NodeSorter(NodeSorter.CHECK2);
		nameSorter2=new NodeSorter(NodeSorter.NAME2);
		occurSorter2=new NodeSorter(NodeSorter.OCCURRENCE2);
		descSorter2=new NodeSorter(NodeSorter.DESCRIPTION2);
		
		Table table=(Table)viewer.getControl();
		TableColumn dot=table.getColumn(0);
		dot.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				viewer.setSorter(null);
				viewer.refresh();
				viewer.getControl().setRedraw(true);
			}
		});		

		TableColumn check=table.getColumn(0);  // check for hide
		check.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(checkSorter);
					clicked=false;
				} else {
					viewer.setSorter(checkSorter2);
					clicked=true;					
				}
				viewer.refresh();
				viewer.getControl().setRedraw(true);
			}
		});

		
		TableColumn name=table.getColumn(1);
		name.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(nameSorter);
					clicked=false;
				} else {
					viewer.setSorter(nameSorter2);
					clicked=true;					
				}
				viewer.refresh();
				viewer.getControl().setRedraw(true);
			}
		});
		TableColumn occurrence=table.getColumn(2);
		occurrence.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(occurSorter);
					clicked=false;
				} else {
					viewer.setSorter(occurSorter2);
					clicked=true;					
				}
				viewer.refresh();
				viewer.getControl().setRedraw(true);
			}
		});
		TableColumn description=table.getColumn(3);
		description.addSelectionListener(new SelectionAdapter() {
			private boolean clicked=false;
			public void widgetSelected(SelectionEvent e) {
				viewer.getControl().setRedraw(false);
				if (clicked) {
					viewer.setSorter(descSorter);
					clicked=false;
				} else {
					viewer.setSorter(descSorter2);
					clicked=true;					
				}
				viewer.refresh();
				viewer.getControl().setRedraw(true);
			}
		});
		
	}
	private void displayNextTree(UniquePath top) {
		if (top!=null)
			editor.getCurrentEditor().refresh(top);
		editor.getCurrentEditor().getViewer().setSelection(new StructuredSelection(top));
		if (SeatUtil.getActivePart()!=this)
			return;		
		if (!((PartialTree)editor.getCurrentEditor().getViewer().getTree()).isIncluded(top))
			SeatUtil.showMessage("The matched node ["+top.getFullQualifiedName()+"] is not currently visible!");		
	}
	// Extra hooks for each action
	public void doubleClicked(){}
	public void nextActionClicked(){}
	public void prevActionClicked(){}
	public void sortActionClicked(){}
	public void deleteActionClicked(){}	
	public void deleteAllActionClicked(){}

	public void connectEditor(MultiTraceEditor meditor) {
		// TODO Auto-generated method stub
		Assert.isTrue(meditor!=null);
		editor=meditor;
		editor.setModelView(this);
		list=editor.getCurrentEditor().getExploration().getModelList();
//		TraceState ts=editor.getTrace().getState();
//		list.clear();
//		list.addAll(ts.getPackageList());
//		list.addAll(ts.getClassList());
//		list.addAll(ts.getModelNodeList());
	}
		
	public void disconnectEditor() {
		if (editor!=null)
			editor.setModelView(null);
	}		
		
	/* (non-Javadoc)
	 * @see seat.ui.views.MethodView#promoteActionCliked(seat.model.core.dynamic.Node)
	 */
	public void promoteActionCliked(Node node) {
		// TODO Auto-generated method stub
		String methodFullName=node.getLabel(); // full name includes pkg
		SeatUtil.addToUtilityDepot(methodFullName);					
		SeatUtil.showMessage("[" + methodFullName+"] is promoted to global!");
	}

	
	/**
	 * Path in the UI Tree with their model node as the given one will be all
	 * highlighted in a predefiend color.
	 * @param node
	 */
	public void setHighLightedNode(Node node) {
		// TODO Auto-generated method stub
		editor.getCurrentEditor().getViewer().setHighlightedNode(node);
		editor.refresh();		
		return;
	}
	
	/**
	 * Shows or Hides UI tree nodes after applying some algorithms.
	 * @param hides
	 * @param shows
	 */
	public void hideAndShowNode(Object[] hides, Object[] shows) {
		if ((hides!=null)&&(hides.length>0)) {
			for (int i=0;i<hides.length;i++) {
				AlgorithmShowHide(hides[i],false);
			}
		}
		if ((shows!=null)&&(shows.length>0)) {
			for (int i=0;i<shows.length;i++) {
				AlgorithmShowHide(shows[i],true);
			}
		}
		viewer.getControl().setRedraw(false);
		viewer.refresh();
		if (shows[0]!=null)
			viewer.reveal(shows[0]);
		else if (hides[0]!=null)
			viewer.reveal(hides[0]);
		viewer.getControl().setRedraw(true);
		updateTitle();
		updateActionStatus();

		editor.refresh();		
	}
	
	// Shows or Hides a model node by running an algorithm
	private void AlgorithmShowHide(Object model, boolean show) {
		viewer.setChecked(model,!show);
		if (show)
			filteredElements.remove(model);
		else
			filteredElements.add(model);			
	}
	
	/**
	 * Makes user selected nodes visible in this viewer and refreshes editor ro reflects changes
	 * @param objs
	 */
	public void showUserSelectedNodes(Object[] objs) {
		usrSelectedShowHide(objs,true);
	}
	
	/**
	 * Makes user selected nodes invisible in this viewer and refreshes editor ro reflects changes
	 */
	public void hideUserSelectedNodes(Object[] objs) {
		usrSelectedShowHide(objs,false);
	}

	private void usrSelectedShowHide(Object[] objs, boolean show) {
		if ((objs==null)||(objs.length==0)) return;
		for (int i=0;i<objs.length;i++) {
			usrSelectedShowHide(objs[i],show);
		}
		viewer.getControl().setRedraw(false);
		viewer.refresh();
		if (objs[0]!=null)
			viewer.reveal(objs[0]);
		viewer.getControl().setRedraw(true);
		updateTitle();
		updateActionStatus();

		editor.refresh();		
	}
	
	// Shows or Hides a model node by user interaction
	private void usrSelectedShowHide(Object model, boolean show) {
		if (model instanceof Node) {
			Node node=(Node)model;
			if (show) // default is: check for hide
				node.show();
			else
				node.hide(SeatUtil.RESERVED_ALGIDX_FILTER);		
		} else if (model instanceof StructuralElement) {  // include both pck and cls 
			String name=((StructuralElement)model).getName();
			for (int i=0;i<list.size();i++) {
				Object obj=list.get(i);
				if (obj instanceof Node) {
					Node node=(Node)obj;
					String nodeLabel=node.getLabel();
					if (nodeLabel.startsWith(name+".")) {
						if (show) // default is: check for hide
							node.show();
						else
							node.hide(SeatUtil.RESERVED_ALGIDX_FILTER);
						viewer.setChecked(node,!show);								
					}
				}
			}
		}
		viewer.setChecked(model,!show);
		if (show) {
			filteredElements.remove(model);			
		}
		else {
			filteredElements.add(model);			
		}
	}
	
	/**
	 * Returns active editor that this view reflects 
	 * @return
	 */
	public MultiTraceEditor getEditor() {
		return editor;
	}
	
	public Viewer getViewer() {
		return viewer;
	}
	
	public ArrayList getFilteredElements() {
		return filteredElements;
	}

}
