package seat.ui.views;

import java.util.ArrayList;

import seat.model.core.dynamic.Node;
import seat.model.core.structure.MethodElement;
import seat.ui.tree.partial.PartialTree;
import seat.ui.tree.partial.UniquePath;

/**
 * @author lfu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PathGroup {
	private Node modelNode;
	private int total;
	private int lastIndex;	
	
	private ArrayList revEdges=new ArrayList();
	private int tempCounter=0;
	private boolean found=false;
	PartialTree partialTree;
	public PathGroup(PartialTree partialTree,Node node) {
		this.partialTree=partialTree;
		this.modelNode=node;
		computeTotal();
		this.lastIndex=0;
	}
	
	public String getName() {
		return modelNode.getLabel();
	}
	
	public int getOccurrence() {
		return (total>0)?total:1;
	}
	public String getDescription() {
		MethodElement me=(MethodElement)modelNode.getBehaviouralElement();
		return me.getDescription();	
	}

	public UniquePath next(){
		if (hasNext()) { // find it
			revEdges.clear();
			found=false;
			tempCounter=0;
			traverseAndGatherNext(modelNode);
		} 
		// re-order data from reversed;
		if (revEdges.size()==0)
			return null;
		ArrayList edges=new ArrayList();
		for (int i=revEdges.size()-1;i>=0;i--)
			edges.add(revEdges.get(i));
		return new UniquePath(edges);	
	}
	
	public UniquePath previous(){
		if (hasPrevious()) {
			revEdges.clear();
			found=false;
			tempCounter=0;
			traverseAndGatherPrev(modelNode);			
		}
		// re-order data from reversed 
		if (revEdges.size()==0)
			return null;
		ArrayList edges=new ArrayList();
		for (int i=revEdges.size()-1;i>=0;i--)
			edges.add(revEdges.get(i));
		return new UniquePath(edges);	
	}

	// id : 0~total-1 inclusive
	public UniquePath find(int id) {
		if ((id<0)||(id>total))
			return null;
		lastIndex=id;
		return next();
	}

	public boolean hasNext(){
		return (lastIndex<total)?true:false;
	}
	public boolean hasPrevious(){	
		return (lastIndex>1)?true:false;
	}
	
	public static int count(Node node) {
		Node parent=null;
		int counter=0;
		for (int i=0;i<node.getNumberOfParents();i++) {
			parent=node.getParentAt(i);
			if (parent.isRoot())
				counter++;
			else 
				counter+=count(parent);
		}
		return counter;
	}
	private void computeTotal() {
		traverse(modelNode);
	}
	
	private void traverse(Node node) {
		Node parent=null;
		for (int i=0;i<node.getNumberOfParents();i++) {
			parent=node.getParentAt(i);
			if (parent.isRoot())
				total++;
			else 
				traverse(parent);
		}
	}

	private void traverseAndGatherNext(Node node) {
		if (found) return;
		Node parent=null; 
		for (int i=0;i<node.getNumberOfParents();i++) {
			if (found) return;
			parent=node.getParentAt(i);
			revEdges.add(node.getParentEdge(i));
			if (parent.isRoot()) {
				tempCounter++;
				if (tempCounter==lastIndex+1) { // found
					revEdges.add(partialTree.virtualEdge);//new Edge(null,parent)); // add the dumb edge
					lastIndex++;
					tempCounter=0;
					found=true;
					return;
				} else
					revEdges.remove(revEdges.size()-1);  // remove last obj
			}
			else
				traverseAndGatherNext(parent);
		} 
		if (!found)
			revEdges.remove(revEdges.size()-1);  // remove last obj		
	}

	private void traverseAndGatherPrev(Node node) {
		if (found) return;
		Node parent=null;
		for (int i=0;i<node.getNumberOfParents();i++) {
			if (found) return;
			parent=node.getParentAt(i);
			revEdges.add(node.getParentEdge(i));
			if (parent.isRoot()) {
				tempCounter++;
				if (tempCounter==lastIndex-1) {
					revEdges.add(partialTree.virtualEdge);//new Edge(null,parent)); // add the dumb edge
					lastIndex--;
					tempCounter=0;
					found=true;
					return;
				}
				else
					revEdges.remove(revEdges.size()-1);  // remove last obj
			}
			else
				traverseAndGatherPrev(parent);
		}
		if (!found)
			revEdges.remove(revEdges.size()-1);  // remove last obj				
	}
	
	/**
	 * @return
	 */
	public Node getNode() {
		return modelNode;
	}

}
