/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package seat.ui.views;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.search.ui.ISearchPage;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.search.ui.SearchUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IEditorPart;

import seat.SeatUtil;
import seat.model.core.dynamic.Node;
import seat.ui.tree.viewer.MultiTraceEditor;


/**
 * TraceSearchPage
 */
public class TraceSearchPage extends DialogPage implements ISearchPage {
	private Combo searchWordCombo = null;
	private Button fExactMatch;
	private boolean fIsExactMatch;
	
	private ISearchPageContainer scontainer = null;
	private boolean firstTime = true;

	private ArrayList searchResult=new ArrayList();
	private static java.util.List previousSearchQueryData =
		new java.util.ArrayList(20);
	private ArrayList distinct;
	
	/**
	 * Search Page
	 */
	public TraceSearchPage() {
		super();
	}
	public void createControl(Composite parent) {
		Font font = parent.getFont();
		Composite control = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		control.setLayout(layout);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		control.setLayoutData(gd);
		// Search Expression
		Label expressionLabel = new Label(control, SWT.LEFT);
		expressionLabel.setFont(font);
		expressionLabel.setText("Search string that appears in the name of nodes:");
		// Pattern combo
		searchWordCombo = new Combo(control, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = convertWidthInCharsToPixels(30);
		searchWordCombo.setLayoutData(gd);
		searchWordCombo.setFont(font);
		searchWordCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (searchWordCombo.getSelectionIndex() < 0)
					return;
				int index =previousSearchQueryData.size()-1	- searchWordCombo.getSelectionIndex();				
			}
		});
		searchWordCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updateSearchButtonState();
			}
		});
		
		IEditorPart active=SeatUtil.getActiveEditor();
		if (active instanceof MultiTraceEditor) {
			ArrayList all=((MultiTraceEditor)active).getTrace().getState().getModelNodeList();   // all nodes for now!!
			distinct=new ArrayList();
			for (int i=0;i<all.size();i++) {
				String label=((Node)all.get(i)).getLabel();
				if (!distinct.contains(label))
					distinct.add(label);
			}
			for (int i=0;i<distinct.size();i++)
				searchWordCombo.add((String)distinct.get(i));
		}
		// Ignore case checkbox		
		fExactMatch= new Button(control, SWT.CHECK);
		fExactMatch.setText("Only Match Method Name");
		gd= new GridData();
		fExactMatch.setLayoutData(gd);
		fExactMatch.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				fIsExactMatch= fExactMatch.getSelection();
			}
		});

		setControl(control);
	}
	/**
	 * @see ISearchPage#performAction()
	 */
	public boolean performAction() {
		
		final String comboString=searchWordCombo.getText();
		final boolean exact=fIsExactMatch;
		if (!previousSearchQueryData.contains(comboString)&&!distinct.contains(comboString))
			previousSearchQueryData.add(comboString);

		try {
			SearchUI.activateSearchResultView();
			scontainer.getRunnableContext().run(
				true,
				true,
				new TraceSearchOperation(comboString,fIsExactMatch,searchResult));
		} catch (InvocationTargetException ex) {
			return false;
		} catch (InterruptedException e) {
			return false;			
		} 
		return true;

//		boolean dontCancel = false;
//		do {
//			try {
//				SearchUI.activateSearchResultView();
//				scontainer.getRunnableContext().run(
//					true,
//					true,
//					new SearchOperation(searchQueryData));
//					dontCancel=false;
//			} catch (InvocationTargetException ex) {
//				return false;
//			} catch (InterruptedException e) {
//				dontCancel = true;
//			}
//		} while (dontCancel);
//		return true;
//			
//			
////		try {
//////			SearchUI.activateSearchResultView();
////			scontainer.getRunnableContext().run(
////				true,
////				true,
////				new TraceSearchOperation(comboString,fIsExactMatch,searchResult));
////		} catch (InvocationTargetException ex) {
////			return false;
////		} catch (InterruptedException e) {
////			return false;			
////		} 
//
//		// prepare to gather search results
//		IEditorPart active=SeatUtil.getActiveEditor();
//		if (!(active instanceof MultiTraceEditor))
//			return true;
//			
//		final TraceSearch searchResult=new TraceSearch((MultiTraceEditor)active);
////		searchResult.gatherAllMatched(comboString,exact);				
//	
//		try {
//			scontainer.getRunnableContext().run(
//				false,
//				true,
//				new IRunnableWithProgress() {
//					public void run(IProgressMonitor monitor) {						
//						monitor.beginTask("Searching for ["+comboString+"] ...",100);
//						searchResult.gatherAllMatched(comboString,exact,monitor);				
//						SearchView sv=(SearchView)SeatUtil.switchToView("seat.ui.views.SearchView");
//						sv.setSearch(searchResult);	
//						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().bringToTop(sv);
//						monitor.done();
//					}
//				});
//		} catch (InvocationTargetException ex) {
//			return false;
//		} catch (InterruptedException e) {
//			return false;			
//		} 
//		return true;
	}
	public void setContainer(ISearchPageContainer container) {
		scontainer = container;
	}
	/*
	 * Implements method from IDialogPage
	 */
	public void setVisible(boolean visible) {
		if (visible && searchWordCombo != null) {
			if (firstTime) {
				firstTime = false;
				// Set item and text here to prevent page from resizing
				int count=previousSearchQueryData.size();
				String[] patterns = new String[count];
				for (int i = 0; i < count; i++) {
					patterns[i]=(String)previousSearchQueryData.get(count - 1 - i);
					searchWordCombo.add(patterns[i],0);
				}
//				searchWordCombo.setItems(patterns);
			}
			searchWordCombo.setFocus();
			updateSearchButtonState();
		}
		super.setVisible(visible);
	}
	
	

	private void updateSearchButtonState() {
		boolean searchWordValid =
			searchWordCombo.getText() != null
				&& searchWordCombo.getText().trim().length() > 0;

		scontainer.setPerformActionEnabled(searchWordValid);
	}
}
